﻿'┌───────────┐
'│DLL 『ダイアルボタン』├───────────┐
'└┬──────┬───┘Programmed by はりまお│
'  │[制作環境]  └───────────┬───┘
'  │CPU  : Intel Core i7-4770           │
'  │            (@3.40GHz * 4 core * HT)│
'  │RAM  : 8GB                          │
'  │OS   : Windows 10 Pro  (21H2, 64bit)│
'  │IDE  : Visual Studio Community 2022 │
'  │Lang : Visual Basic                 │
'  │CLI  : .NET Framework 4.8           │
'  └──────────────────┘

#Region "掲示板"
'┌────┐
'│技術要綱├──────────────────────
'└┬───┘
'  │『高（低）分解能ホイールへの対応』
'  │○ホイールの回転量は、《.MouseWheel》イベントの《e.Delta》で知ることができます。1ノッチ分で増減する差分値は、基本的に120（これを「基準デルタ値」と勝手に呼称する）で、回転方向により符号が反転します。高速に連続回転した場合は、240とか360とかが得られます。
'  │○なぜ"120"なのかというと、もし高（低）分解能のホイールが搭載されたマウスが登場した場合に対応するためだそうです（それがどういう物なのか、実際に存在するのかは知りません）。要するに、120という値は絶対的な値ではないということ。
'  │●この状況に対応するために、以下の方法を採りました。
'  │  ├ 1. 基準デルタ値は、≪SystemInformation.MouseWheelScrollDelta≫を参照する。ちなみに、≪SystemInformation≫には、このほかにもいろいろなシステム環境に関する情報が格納されています。（Windowsでプログラムを書くようになってから、この値をどこで知ればいいのかわからなくて困ってた。でも絶対どこかにあるとは思ってたんだ。）
'  │  └ 2. 《.MouseWheel》イベントで得られる《e.Delta》値を累積して、基準デルタ値を満たした分を回転として処理し、余った分を次回以降に持ち越す。（もしかして、この処理をOS側でやってくれているのかも！？）
'  │※「ノッチ」とは、ホイールを回した時に“カチッカチッ”となる引っ掛かりのことです。正式名称は、知りません。
'  │
'  │『無段ホイールへの対応』
'  │○世の中のホイールには、回転する時に、ノッチが有る物（有段）と無い物（無段）が存在します。
'  │○無段ホイールでは、1ノッチ分の回転をユーザが認識するのは困難です。今回のように、ホイールを決まった分だけ回転させる必要がある場合、これでは困ってしまいます。
'  │●これに対応するために、以下の方法を採りました。
'  │  ├ 1. 基本的に、一方向に連続する回転は、1ノッチと考える。
'  │  ├ 2. 同じ方向に複数回回転する場合のために、回転が分離しているとを判断するための間隔（閾時間）を決める。「“回転→指を戻す→回転”という流れ」や「ある程度ゆっくり回しても過敏に反応しない」ことを考えると、250ミリ秒でちょうど良いと思う。
'  │  ├ 3. 前回の回転から今回の回転までを、《Stopwatch》クラスを使って計測し、閾時間以上空いていれば別の回転と判断する。
'  │  └ 4. ただし、「ダイヤルボタンにマウスカーソルが入った直後」や「回転方向が反転した時」は、閾時間に関係なく回転を分離する。
'  │・タッチパネル式のホイールにもこの処理で対処できると思うが、指を置いたり離したりする際に回転を認識してしまうようなことがあると上手く行かないかもしれない。そういうマウスを持っていないので検証できず。
'  │※「無段ホイール」とは、作者が勝手に付けた名前です。
'  │
'  │『無段ホイール モードの使い分け』
'  │○このフラグを切り替えると、入力イメージが以下のように変わります。
'  │・無効・・・主にノッチがあるホイールを使用する時。「上に3 下に1 上に2 …」と入力したい時に向く。実際のダイアル錠により近いかも（使ったことないから分からないけど）。
'  │・有効・・・主にノッチがない（感知し難い）ホイールを使用する時。「上下上上下…」と入力したい時に向く。大胆に入力できるので、こっちの方が簡単かも。
'  │
'  │『その他』
'  │○ステップが上限を超えた時に外部の音声ファイルを鳴らしたい場合に対応するため、《Alarm》プロパティを設けている。
'  │○状況に応じて前景色を変化させるのを、外部から行うのは大変なので、内部で強制的に行う。その代わり、色は選択可能とする。
'  └────────────
'┌───────┐
'│制作・改良履歴├───────────────────
'└┬──────┘
'  │「Ver.a.b.c.d」（下位の変化は上位の変化に含める。"."は小数点ではなく各要素の区切り。）
'  │  a = 全面改訂 例）新規プロジェクトを興したなど。（0=テスト,サンプル版）
'  │  b = 外向変更 例）機能の追加・削除や使用方法の変更など。
'  │  c = 内向変更 例）使用上の影響が軽微なバグ修正,挙動改善,外観調整など。バグフィックス
'  │  d = 微修整   例）本質部の結果に無影響なコード（手順,式,名称等）の変更など。リファクタリング
'  │
'  │2022年01月27日：Ver.1.1.1.2 ─ d.開発・動作環境を移行。
'  │                                 →Visual Studio 2015 → 2022
'  │                                 →.NET Framework 4.5.2 → 4.8
'  │2017年08月23日：Ver.1.1.1.1 ─ d.開発・動作環境を移行。
'  │                                 →Windows 10 + Visual Studio 2015
'  │                                 →.NET Framework 4.5.2
'  │2015年04月06日：Ver.1.1.1.0 ─ c.フラグに《BitVector32》構造体を使用。（2個のフラグしかないので有効ではないだろうけど、知ったものは使ってみたい。ついでに、《BitVector32.CreateMask》も使った。）
'  │2015年04月05日：Ver.1.1.0.0 ┬ b.マウスカーソルが出入しようとしているのを知らせるイベントを追加した。≪MouseIn≫イベントは≪MouseEnter≫イベント処理の最初に、≪MouseOut≫イベントは≪MouseLeave≫イベント処理の最後に発生させる。
'  │                            └ c:マウスカーソルが乗っている間は、フォーカスを放さないようにした。
'  │2014年04月17日：Ver.1.0.4.0 ┬ c.ホイールの回転1目盛り分で増減する差分値をシステム環境情報から取得するようにした。（≪SystemInformation≫の存在を初めて知った。これは画期的！）
'  │                            └ Windows 8.1へ移行。
'  │2013年06月12日：Ver.1.0.3.1 ─ d.アセンブリ情報の説明を変更。
'  │2012年11月09日：Ver.1.0.3.0 ┬ ステップ数が最大数に達した時に、注意色になるようにした。
'  │                            └ 全使用色を変更可能にした。警告音の使用を選択可能にした。
'  │2012年11月04日：Ver.1.0.2.0 ─ 入力に対する基本的な反応を全て内包（利用するか選択可能）。
'  │2012年11月03日：Ver.1.0.1.1 ─ 無意味な処理があったので削除。
'  │2012年11月02日：Ver.1.0.1.0 ┬ 無段ホイールに対応。これに伴い、《NotchPattern》から《StepPattern》に変更。
'  │                            └ ステップパターンの入力プロセスを変更。
'  │2012年08月26日：Ver.1.0.0.4 ─ “やり直し”から“取り消す”に変更。“やり直し”だと、「入力しなおせ！」と言われているみたいなので。
'  │2012年02月25日：Ver.1.0.0.3 ─ キーでのダイアル入力に応急的に対応。
'  │2012年02月13日：Ver.1.0.0.2 ┬ 《NotchPattern》プロパティのSet処理を変更。
'  │                            └ ホイール回転認識処理手順の変更。
'  │2012年02月10日：Ver.1.0.0.1 ┬ 高解像度ホイールへ対応。※ただし、高解像の環境を持っていないので、これで解決できているのか不明。それに、1ノッチ分が判断し辛いので意味がないかも。
'  │                            └ マウスカーソルが乗っていない時、ホイールの回転を無視するのを失念していた。
'  │2012年02月07日：Ver.1.0.0.0 ─ とりあえず完成
'  │2012年02月03日：Ver.0.0.0.0 ─ 制作開始
'  └────────────
#End Region



Imports System.ComponentModel



''' <summary>
''' ダイアルボタン
''' </summary>
''' <remarks></remarks>
<DefaultProperty("StepPattern"),
 DefaultEvent("StepPatternChanged")> _
Public Class DialButton

    Inherits Button 'Buttonを継承



    '==============================================================



#Region "イベント定義"

    ''' <summary>
    ''' ステップパターンに変化があった
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    <Category("ステップパターン"),
     Description("ステップパターンが変更された時に発生します。")> _
    Public Event StepPatternChanged(ByVal sender As Object, ByVal e As EventArgs)

    ''' <summary>
    ''' ステップ数が上限を超えた
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    <Category("ステップパターン"),
     Description("ステップ数が上限を超えたときに発生します。")> _
    Public Event StepPatternOverflow(ByVal sender As Object, ByVal e As EventArgs)


    '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


    ''' <summary>
    ''' マウスが領域内に入ろうとしている
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    <Category("マウス"),
     Description("マウスがコントロール内に入ろうとしています。")> _
    Public Event MouseIn(ByVal sender As Object, ByVal e As EventArgs)

    ''' <summary>
    ''' マウスが領域内から出ようとしている
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    <Category("マウス"),
     Description("マウスがコントロール内から出ようとしています。")> _
    Public Event MouseOut(ByVal sender As Object, ByVal e As EventArgs)

#End Region



    '==============================================================



#Region "プロパティ定義"

    Private _StepPattern As String = "" '《StepPattern》用の記憶場所



    '--------------------------------------------------------------



    ''' <summary>テキストを取得・設定します</summary>
    <Browsable(False),
     ReadOnlyAttribute(True),
     DefaultValue("")> _
    Public Overloads Property Text As String

        Get
            Return MyBase.Text
        End Get

        Set(value As String)
            MyBase.Text = value
        End Set

    End Property

    ''' <summary>ダイアルのステップパターン</summary>
    <Category("ステップパターン"),
     Description("ダイアルのステップパターンです。"),
     DefaultValue("")> _
    Public Property StepPattern As String

        Get
            Return _StepPattern.ToUpper
        End Get

        Set(value As String)

            Dim befor As String = _StepPattern

            '=== ステップ数超過のチェックと調整 ===
            If value.Length > StepMaxLength Then
                _StepPattern = Strings.Left(value, StepMaxLength)
                RaiseEvent StepPatternOverflow(Me, New EventArgs)
            Else
                _StepPattern = value
            End If

            '=== ステップパターン変化チェック ===
            If _StepPattern <> befor Then
                RaiseEvent StepPatternChanged(Me, New EventArgs)
            End If

        End Set

    End Property

    ''' <summary>無段ホイール モード</summary>
    <Category("動作"),
     Description("無段ホイール モードを使用するかどうかを設定する。"),
     DefaultValue(False)> _
    Public Property SlipWheel As Boolean = False

    ''' <summary>回転分離閾時間</summary>
    <Category("動作"),
     Description("回転を分離する閾時間をミリ秒で設定する。"),
     DefaultValue(250)> _
    Public Property RollSeparateTime As Integer = 250

    ''' <summary>ステップ数の上限規制</summary>
    <Category("動作"),
     Description("ステップパターンに入力できる最大ステップ数を設定します。"),
     DefaultValue(32767)> _
    Public Property StepMaxLength As Integer = 32767

    ''' <summary>警告音</summary>
    <Category("動作"),
     Description("最大ステップ数を超えて入力しようとした時に警告音を鳴らすかどうかを設定します。"),
     DefaultValue(True)> _
    Public Property Alarm As Boolean = True

    ''' <summary>待機色</summary>
    <Category("色"),
     Description("入力対象ではない時の前景色。")> _
    Public Property StaticColor As Color = SystemColors.ControlText

    ''' <summary>点滅色1</summary>
    <Category("色"),
     Description("ホイールを回した時に、前景色を点滅させるための1つ目の色。")> _
    Public Property StepColor1 As Color = Color.Lime

    ''' <summary>点滅色2</summary>
    <Category("色"),
     Description("ホイールを回した時に、前景色を点滅させるための2つ目の色。")> _
    Public Property StepColor2 As Color = Color.LimeGreen

    ''' <summary>注意色</summary>
    <Category("色"),
     Description("最大ステップ数に達した時の前景色。")> _
    Public Property AttentionColor As Color = Color.Gold

    ''' <summary>警告色</summary>
    <Category("色"),
     Description("最大ステップ数超えて入力しようとした時の前景色。")> _
    Public Property CautionColor As Color = Color.Red

#End Region



    '==============================================================



    ''' <summary>上方回転</summary>
    Private Const RollUp As String = "U"

    ''' <summary>下方回転</summary>
    Private Const RollDown As String = "D"

    '--------------------------------------------------------------

    ''' <summary>マウスカーソルが乗っているかを示すフラグ用マスク</summary>
    Private ReadOnly BitMaskHovering As Integer '値は《New》で確定する。

    ''' <summary>マウスカーソルが乗った直後であることを示すフラグ用マスク</summary>
    Private ReadOnly BitMaskFirstStep As Integer '値は《New》で確定する。



    '==============================================================



    ''' <summary>フラグ</summary>
    Private Flags As New System.Collections.Specialized.BitVector32(0)

    ''' <summary>前回の残りDelta（高解像度ホイールへの対応）</summary>
    Private RemnantDelta As Integer = 0

    ''' <summary>回転分離用ストップウォッチ（無段ホイールへの対応）</summary>
    Private RollSeparateStopwatch As New Stopwatch



    '==============================================================



    ''' <summary>
    ''' コンストラクタ
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()

        MyBase.New()

        '=== フラグ用マスク作成 ===
        BitMaskHovering = System.Collections.Specialized.BitVector32.CreateMask
        BitMaskFirstStep = System.Collections.Specialized.BitVector32.CreateMask(BitMaskHovering)

        '=== 表示初期化 ===
        TextChange()

    End Sub

    Private Sub DialButton_MouseEnter(sender As Object, e As System.EventArgs) Handles Me.MouseEnter

        '=== 準備 ===
        RaiseEvent MouseIn(Me, New EventArgs)

        '=== マウスが入った ===
        Flags(BitMaskHovering) = True
        Flags(BitMaskFirstStep) = True
        RemnantDelta = 0
        TextChange()

        Me.Select()
        Me.Font = New Font(Me.Font, Me.Font.Style Or FontStyle.Bold)

    End Sub

    Private Sub DialButton_MouseLeave(sender As Object, e As System.EventArgs) Handles Me.MouseLeave

        '=== マウスが出た ===
        Flags(BitMaskHovering) = False
        TextChange()

        RollSeparateStopwatch.Stop()

        Me.ForeColor = StaticColor
        Me.Font = New Font(Me.Font, Me.Font.Style Xor FontStyle.Bold)

        '=== 後始末 ===
        RaiseEvent MouseOut(Me, New EventArgs)

    End Sub

    Private Sub DialButton_Leave(sender As Object, e As System.EventArgs) Handles Me.Leave

        '=== マウスカーロスが乗っている間はフォーカスを放さない ===
        If Flags(BitMaskHovering) Then Me.Select()

    End Sub

    Private Sub DialButton_Click(sender As Object, e As System.EventArgs) Handles Me.Click

        '=== やり直し ===
        StepPattern = ""
        RemnantDelta = 0

    End Sub

    Private Sub DialButton_MouseWheel(sender As Object, e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseWheel

        '=== ホイール回転 ===
        If Flags(BitMaskHovering) Then

            If SlipWheel Then

                '--- 無段（つるつる）ホイール用 ---
                '一定方向に一度に回転した分をまとめて1ノッチとみなす。
                If RollSeparateStopwatch.ElapsedMilliseconds > RollSeparateTime OrElse
                   Flags(BitMaskFirstStep) OrElse
                   (StepPattern.Last = RollUp AndAlso e.Delta < 0 OrElse StepPattern.Last = RollDown AndAlso e.Delta > 0) Then
                    '「回転を分離する時間以上に間隔が空いた時」,「ダイアルボタン上にマウスが入った直後」,「回転方向が変わった時」だけ入力を認める。
                    '未入力時に直近のステップを抽出する（"StepPattern.Last"）とエラーになるので、その前に《Flags(FlagMasks.FirstStep)》のチェックを必ず行う（未入力＝マウスが乗った直後）。

                    InputNotch(SystemInformation.MouseWheelScrollDelta * Math.Sign(e.Delta))
                    Flags(BitMaskFirstStep) = False

                End If

                RollSeparateStopwatch.Restart() '計測時間をリセット

            Else

                '--- 有段（かくかく）ホイール用 ---
                'ホイールの回転をそのまま入力する。
                InputNotch(e.Delta)

            End If

        End If

    End Sub

    Private Sub DialButton_KeyDown(sender As Object, e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown

        '=== キーでダイアルを操作するとき、強制的にマウスカーソルを移動 ===
        With DirectCast(sender, DialButton)
            Cursor.Position = .PointToScreen(New Point(.Width * 0.6, .Height * 0.6))
        End With

    End Sub

    Private Sub DialButton_KeyUp(sender As Object, e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyUp

        '=== キーでダイアルを回す ===
        '《KeyUp》イベントで行うのは、1ノッチずつ確実にキーを押させるため。
        Dim roll As Integer = 0 '回転方向

        Select Case e.KeyCode
            Case Keys.PageUp
                roll = SystemInformation.MouseWheelScrollDelta
            Case Keys.PageDown
                roll = -SystemInformation.MouseWheelScrollDelta
        End Select

        If roll <> 0 Then InputNotch(roll)

    End Sub

    ''' <summary>
    ''' ノッチ入力をステップパターンに登録
    ''' </summary>
    ''' <param name="Delta">回転情報</param>
    ''' <remarks></remarks>
    Private Sub InputNotch(Delta As Integer)

        '=== ステップパターン登録 ===
        If Math.Sign(RemnantDelta) <> Math.Sign(Delta) Then RemnantDelta = 0 '今までと今回の回転方向が逆の場合は、累積値をクリアする。
        RemnantDelta += Delta '前回の残りに今回分を足す。
        Dim data As Integer = RemnantDelta \ SystemInformation.MouseWheelScrollDelta
        StepPattern &= Strings.StrDup(Math.Abs(data), IIf(Math.Sign(data) < 0, RollDown, RollUp))

        RemnantDelta = RemnantDelta Mod SystemInformation.MouseWheelScrollDelta '半端分（WHEEL_DELTAに満たない分）を記憶

    End Sub

    Private Sub DialButton_StepPatternChanged(sender As Object, e As System.EventArgs) Handles Me.StepPatternChanged

        Static switch As Boolean = False '色切り替えスイッチ

        '=== 状態の変化を色でお知らせ ===
        If StepPattern = "" Then

            Me.ForeColor = StaticColor

        ElseIf StepPattern.Length = StepMaxLength Then

            Me.ForeColor = AttentionColor

        ElseIf Flags(BitMaskHovering) Then

            switch = Not switch
            Me.ForeColor = IIf(switch, StepColor1, StepColor2)

        End If

        '=== 表示変更 ===
        TextChange()

    End Sub

    Private Sub DialButton_StepPatternOverflow(sender As Object, e As System.EventArgs) Handles Me.StepPatternOverflow

        '=== もう満杯だよ！ ===
        DirectCast(sender, DialButton).ForeColor = CautionColor
        If Alarm Then My.Computer.Audio.PlaySystemSound(Media.SystemSounds.Beep)

    End Sub

    ''' <summary>
    ''' 《Text》プロパティを変更する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub TextChange()

        If StepPattern = "" Then

            Text = "未入力"

        ElseIf StepPattern.Length = StepMaxLength Then

            Text = "満杯"

        Else

            If Flags(BitMaskHovering) Then
                Text = "取り消す"
            Else
                Text = "入力済み"
            End If

        End If

    End Sub

End Class
