﻿'┌────┐
'│技術要綱├──────────────────────
'└┬───┘
'  │『概要』
'  │・SendInput APIを使って、キーボードやマウスの操作を再現する。
'  │・ハードウェアへの働き掛けも可能であると思われるが、何ができるのかは不明である。
'  │・今回のプロジェクトではマウス操作しか行わないので、過剰な実装となっているが、サンプルとしての役割もあるので全部入りで収録しておく。
'  │
'  │『リファレンス』
'  │・http://nonsoft.la.coocan.jp/SoftSample/CS.NET/SampleSendInput.html
'  │・http://hanatyan.sakura.ne.jp/
'  │  →http://hanatyan.sakura.ne.jp/vb2005/vb2013keyboard03.htm
'  │  →http://hanatyan.sakura.ne.jp/vb2005/vb2013mouse02.htm
'  │・上記サイトを混ぜ合わせて作った。
'  │
'  │『絶対座標によるマウスカーソルの移動』
'  │・マウスカーソルの移動先を絶対座標で指定する場合、その座標は(0,0)-(65535,65535)で指定しなければならないみたい。
'  │  →そもそも、65535って何？
'  └────────────





Imports System.Runtime.InteropServices



''' <summary>
''' SendInput APIを使う。
''' </summary>
Public Class SendInputter

    ''' <summary>
    ''' 操作対象
    ''' </summary>
    Public Enum Destination As Integer

        ''' <summary>マウス</summary>
        Mouse = 0

        ''' <summary>キーボード</summary>
        Keyboard = 1

        ''' <summary>ハードウェア</summary>
        Haedware = 2

    End Enum



    '=============================================================================================



    ''' <summary>
    ''' マウス操作
    ''' </summary>
    Public Enum MouseEvents As Integer

#Region "カーソル関連"

        ''' <summary>移動</summary>
        CursolMove = &H1

        ''' <summary>絶対座標を指定</summary>
        Absolute = &H8000

        ''' <summary>絶対座標で移動</summary>
        CursolMoveAbsolute = CursolMove Or Absolute

#End Region

#Region "ボタン関連"

        ''' <summary>左ボタン押下</summary>
        ButtonLeftDown = &H2

        ''' <summary>左ボタン解放</summary>
        ButtonLeftUp = &H4

        ''' <summary>右ボタン押下</summary>
        ButtonRightDown = &H8

        ''' <summary>右ボタン解放</summary>
        ButtonRightUp = &H10

        ''' <summary>中央ボタン押下</summary>
        ButtonMiddleDown = &H20

        ''' <summary>中央ボタン解放</summary>
        ButtonMiddleUp = &H40

        ''' <summary>Xボタン押下</summary>
        ButtonXDown = &H100

        ''' <summary>Xボタン解放</summary>
        ButtonXUp = &H200

#End Region

#Region "ホイール関連"

        ''' <summary>ホイール回転</summary>
        Wheel = &H800

#End Region

    End Enum


    ':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


    Public ReadOnly WHEEL_DELTA As Integer = SystemInformation.MouseWheelScrollDelta '120


    ':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


    ''' <summary>
    ''' キーボード操作
    ''' </summary>
    Public Enum KeyboardEvents As Integer

        ''' <summary>キー押下</summary>
        KeyDown = &H0

        ''' <summary>キー解放</summary>
        KeyUp = &H2

        ''' <summary>拡張コード</summary>
        ExtendedKey = &H1

    End Enum



    '=============================================================================================



    ''' <summary>
    ''' マウス操作指定用
    ''' </summary>
    <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Auto)>
    Public Structure MouseInput

        ''' <summary>X座標（絶対座標/相対座標）</summary>
        Public dx As Integer

        ''' <summary>Y座標（絶対座標/相対座標）</summary>
        Public dy As Integer

        ''' <summary>ホイールの回転量</summary>
        Public roll As Integer

        ''' <summary>操作内容を指定するフラグ</summary>
        Public dwFlags As MouseEvents

        ''' <summary>タイムスタンプ(このメンバーは無視されます)</summary>
        Public time As Integer

        ''' <summary>追加情報(このメンバーは無視されます)</summary>
        Public dwExtraInfo As IntPtr

    End Structure


    ':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


    ''' <summary>
    ''' キーボード操作指定用
    ''' </summary>
    <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Auto)>
    Public Structure KeyboardInput

        ''' <summary>仮想キーコード（254の範囲内の値）</summary>
        Public wVk As Short

        ''' <summary>ハードウェアキーのスキャンコード</summary>
        Public wScan As Short

        ''' <summary>操作内容を指定するフラグ</summary>
        Public dwFlags As KeyboardEvents

        ''' <summary>タイムスタンプ(このメンバーは無視されます)</summary>
        Public time As Integer

        ''' <summary>追加情報(このメンバーは無視されます)</summary>
        Public dwExtraInfo As IntPtr

    End Structure


    ':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


    ''' <summary>
    ''' ハードウェア操作指定用
    ''' </summary>
    <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Auto)>
    Public Structure HardwareInput

        Public uMsg As Integer

        Public wParamL As Short

        Public wParamH As Short

    End Structure


    ':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


    ''' <summary>
    ''' 各種操作格納用
    ''' </summary>
    <StructLayout(LayoutKind.Explicit, CharSet:=CharSet.Auto)>
    Public Structure InputUnion

        ''' <summary>マウス操作</summary>
        <FieldOffset(0)> Public mouseOperation As MouseInput

        ''' <summary>キーボード操作</summary>
        <FieldOffset(0)> Public keyboardOperation As KeyboardInput

        ''' <summary>ハードウェア操作</summary>
        <FieldOffset(0)> Public hardwareOperation As HardwareInput

    End Structure


    ':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


    ''' <summary>
    ''' 操作内容集約用
    ''' </summary>
    <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Auto)>
    Public Structure InputSet

        ''' <summary>SendInputの使用目的 0=マウス　1=キーボード　2=ハードウェア</summary>
        Public type As Destination

        ''' <summary>操作内容</summary>
        Public Union As InputUnion

    End Structure


    ':::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


    '※ 64bitでは動かない。
    '32bitでは<FieldOffset(4)>、64bitでは<FieldOffset(8)>にしないといけないらしい。でも実行時の環境に合わせて切り替えるのは難しい。
    'そこで、https://qiita.com/kob58im/items/5a9d909377272d74eefd や https://jinblog.at.webry.info/201604/article_1.html に倣って、InputUnion構造体を設けてやれば64bitでも使えるようになった。

    '''' <summary>
    '''' 操作内容格納用
    '''' </summary>
    '<StructLayout(LayoutKind.Explicit, Size:=28)>
    'Public Structure InputSet

    '    ''' <summary>SendInputの使用目的 0=マウス　1=キーボード　2=ハードウェア</summary>
    '    <FieldOffset(0)> Public type As Destination

    '    ''' <summary>マウス操作内容</summary>
    '    <FieldOffset(4)> Public mouseOperation As MouseInput

    '    ''' <summary>キーボード操作内容</summary>
    '    <FieldOffset(4)> Public keyboardOperation As KeyboardInput

    '    ''' <summary>ハードウェア操作内容</summary>
    '    <FieldOffset(4)> Public hardwareOperation As HardwareInput

    'End Structure



    '=============================================================================================



    ''' <summary>
    ''' キーストロークやマウス操作を再現する。
    ''' </summary>
    ''' <param name="nInputs">入力イベントの数</param>
    ''' <param name="pInputs">挿入する入力イベントの配列</param>
    ''' <param name="cbsize">構造体のサイズ</param>
    ''' <returns>挿入することができたイベントの数を返す。ブロックされている場合は 0 を返す</returns>
    <DllImport("user32.dll", CharSet:=CharSet.Auto)>
    Public Shared Function SendInput(
        ByVal nInputs As Integer,
        ByVal pInputs() As InputSet,
        ByVal cbsize As Integer) As Integer
    End Function

    ''' <summary>
    ''' 仮想キーコード・ASCII値・スキャンコード間でコードを変換する。
    ''' </summary>
    ''' <param name="wCode">仮想キーコードまたはスキャンコード</param>
    ''' <param name="wMapType">実行したい変換の種類</param>
    ''' <returns>スキャンコード、仮想キーコード、ASCII 値のいずれかが返ります。（変換されないときは、0 が返ります。）</returns>
    <DllImport("user32.dll", CharSet:=CharSet.Auto, EntryPoint:="MapVirtualKeyA")>
    Public Shared Function MapVirtualKey(
        ByVal wCode As Integer,
        ByVal wMapType As Integer) As Integer
    End Function





    '|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





    ''' <summary>
    ''' マウスの操作設定を作成する。
    ''' </summary>
    ''' <param name="operation">操作内容</param>
    ''' <returns></returns>
    Public Overloads Shared Function MakeInputSet(operation As MouseInput) As InputSet

        Dim u = New InputUnion With {.mouseOperation = operation}
        Return New InputSet With {.type = Destination.Mouse, .Union = u}

    End Function

    ''' <summary>
    ''' マウスの操作設定を作成する。
    ''' </summary>
    ''' <param name="dwFlags">操作内容フラグ</param>
    ''' <param name="dx">X座標</param>
    ''' <param name="dy">Y座標</param>
    ''' <param name="roll">ホイール回転量</param>
    ''' <returns></returns>
    Public Overloads Shared Function MakeInputSet(dwFlags As MouseEvents, Optional dx As Integer = 0, Optional dy As Integer = 0, Optional roll As Integer = 0) As InputSet

        Dim u = New InputUnion With {.mouseOperation = New MouseInput With {.dx = dx, .dy = dy, .roll = roll, .dwFlags = dwFlags, .time = 0, .dwExtraInfo = IntPtr.Zero}}
        Return New InputSet With {.type = Destination.Mouse, .Union = u}

    End Function

    ''' <summary>
    ''' キーボードの操作設定を作成する。
    ''' </summary>
    ''' <param name="operation">操作内容</param>
    ''' <returns></returns>
    Public Overloads Shared Function MakeInputSet(operation As KeyboardInput) As InputSet

        Dim u = New InputUnion With {.keyboardOperation = operation}
        Return New InputSet With {.type = Destination.Mouse, .Union = u}

    End Function

    ''' <summary>
    ''' キーボードの操作設定を作成する。
    ''' </summary>
    ''' <param name="dwFlags">操作内容フラグ</param>
    ''' <param name="wVk">仮想キーコード</param>
    ''' <param name="wScan">スキャンコード</param>
    ''' <returns></returns>
    Public Overloads Shared Function MakeInputSet(dwFlags As KeyboardEvents, Optional wVk As Short = 0, Optional wScan As Short = 0) As InputSet

        Dim u = New InputUnion With {.keyboardOperation = New KeyboardInput With {.wVk = wVk, .wScan = wScan, .dwFlags = dwFlags, .time = 0, .dwExtraInfo = IntPtr.Zero}}
        Return New InputSet With {.type = Destination.Mouse, .Union = u}

    End Function

    ''' <summary>
    ''' 絶対座標指定用の座標に変換する。
    ''' </summary>
    ''' <param name="position">移動先座標</param>
    ''' <returns></returns>
    Public Shared Function ConvertPosition(position As Point) As Point

        Dim display As Rectangle = Screen.PrimaryScreen.Bounds

        '※WPFで使用する時の注意事項
        '《Forms.Screen》を使うために《System.Windows.Forms》を参照に加える。
        'また、《Forms.Screen.PrimaryScreen.Bounds》は《System.Drawing.Rectangle》なので、《System.Drawing》も参照に追加する必要がある。
        '（WPFでの《Rectangle》は、《System.Windows.Shapes.Rectangle》を指す。）

        Return New Point(65535 * position.X / display.Width, 65535 * position.Y / display.Height)

    End Function

    ''' <summary>
    ''' 操作を送る。
    ''' </summary>
    ''' <param name="operations">操作設定</param>
    ''' <returns></returns>
    Public Shared Function Send(ParamArray operations() As InputSet) As Integer

        Return SendInput(operations.Length, operations, Marshal.SizeOf(GetType(InputSet)))

    End Function

End Class
