﻿Imports System.IO

Namespace SortComparers

    ''' <summary>
    ''' 辞書順比較メソッド
    ''' </summary>
    Public Class DictionaryComparer

        Implements IComparer(Of String)
        Implements IComparer(Of DirectoryInfo)
        Implements IComparer(Of FileInfo)



        Public Function Compare(x As String, y As String) As Integer Implements IComparer(Of String).Compare

            Return String.Compare(x, y)

        End Function

        Public Function Compare(x As DirectoryInfo, y As DirectoryInfo) As Integer Implements IComparer(Of DirectoryInfo).Compare

            Return Compare(x.Name, y.Name)

        End Function

        Public Function Compare(x As FileInfo, y As FileInfo) As Integer Implements IComparer(Of FileInfo).Compare

            Return Compare(x.Name, y.Name)

        End Function

    End Class



    ''' <summary>
    ''' 自然順比較メソッド
    ''' </summary>
    Public Class NaturalComparer

        Implements IComparer(Of String)
        Implements IComparer(Of DirectoryInfo)
        Implements IComparer(Of FileInfo)


        '※ 注意 ※
        '今回、順序判定に用いているAPI"StrCmpLogicalW"なら、エクスプローラのソートと同じ結果になる（と思う）。
        'ただし、StrCmpLogicalWは推移的（【A>B 且つ B>C なら Cは最小である】と推定する）ではないらしく、ソース次第では無限ループに陥る（【A>B 且つ B>C 且つ C>A】が成り立ってしまう）ようです。
        '実アプリに実装する際は、何らかの対処が必要になりそう。

        <Runtime.InteropServices.DllImport("shlwapi.dll", CharSet:=Runtime.InteropServices.CharSet.Unicode, ExactSpelling:=True)>
        Private Shared Function StrCmpLogicalW(x As String, y As String) As Integer : End Function
        'Private Declare Auto Function StrCmpLogicalW Lib "shlwapi.dll" (x As String, y As String) As Integer 'Declareでも呼び出せるけど、古式VBの名残なので潰しが効かない。でもスッキリして好みなんだよなぁ…。



        Public Function Compare(x As String, y As String) As Integer Implements IComparer(Of String).Compare

            Return StrCmpLogicalW(x, y)

        End Function

        Public Function Compare(x As DirectoryInfo, y As DirectoryInfo) As Integer Implements IComparer(Of DirectoryInfo).Compare

            Return Compare(x.Name, y.Name)

        End Function

        Public Function Compare(x As FileInfo, y As FileInfo) As Integer Implements IComparer(Of FileInfo).Compare

            Return Compare(x.Name, y.Name)

        End Function

    End Class



    ''' <summary>
    ''' 作成日時順比較メソッド
    ''' </summary>
    Public Class CreationTimeComparer

        Implements IComparer(Of Date)
        Implements IComparer(Of DirectoryInfo)
        Implements IComparer(Of FileInfo)



        Public Function Compare(x As Date, y As Date) As Integer Implements IComparer(Of Date).Compare

            Return Date.Compare(x, y)

        End Function

        Public Function Compare(x As DirectoryInfo, y As DirectoryInfo) As Integer Implements IComparer(Of DirectoryInfo).Compare

            Return Compare(x.CreationTime, y.CreationTime)

        End Function

        Public Function Compare(x As FileInfo, y As FileInfo) As Integer Implements IComparer(Of FileInfo).Compare

            Return Compare(x.CreationTime, y.CreationTime)

        End Function

    End Class



    ''' <summary>
    ''' 更新日時順比較メソッド
    ''' </summary>
    Public Class LastWriteTimeComparer

        Implements IComparer(Of Date)
        Implements IComparer(Of DirectoryInfo)
        Implements IComparer(Of FileInfo)



        Public Function Compare(x As Date, y As Date) As Integer Implements IComparer(Of Date).Compare

            Return Date.Compare(x, y)

        End Function

        Public Function Compare(x As DirectoryInfo, y As DirectoryInfo) As Integer Implements IComparer(Of DirectoryInfo).Compare

            Return Compare(x.LastWriteTime, y.LastWriteTime)

        End Function

        Public Function Compare(x As FileInfo, y As FileInfo) As Integer Implements IComparer(Of FileInfo).Compare

            Return Compare(x.LastWriteTime, y.LastWriteTime)

        End Function

    End Class

End Namespace
