﻿'┌────┐
'│技術要綱├──────────────────────
'└┬───┘
'  │『参照』
'  │・https://hnx8.hatenablog.com/entry/20130323/1364049814
'  │
'  │『あらまし』
'  │・通常、メニュー項目が多くなって画面外に溢れてしまうと、上下にスクロールボタンが現れ1項目ずつスクロールする。だけど、このUIは流石に辛気臭い。
'  │  （.NET Core系になってからか、昔に比べてスクロールボタンが拡大されている。こちらに移行すれば、若干ではあるが改善されるのだが…。個人的にはスクロールバーを使えるのが理想だけど。）
'  │・ContextMenuStrip（の基底クラスであるToolStripDropDownMenu）は、マウスホイール回転イベントやスクロール用ボタン・メソッドを持っている。
'  │  これらは、非public/protectedだったりPrivateだったりして隠されているけど、リフレクションで無理やり引きずり出して叩けば、なんとかできるんじゃないか…。
'  │
'  │『[PageUp][PageDown]キーによる選択位置の移動』
'  │・SendKeys.Send(Keys.Up)とかSendKeys.Send(Keys.Down)とできたら簡単だけど、なぜか下の階層へフォーカスが移動してしまって上手く行かなかった。
'  │
'  │『スクロールする行数』
'  │・マウスホイールと[PageUp][PageDown]キーによるスクロール行数は、プロパティ化しても良かったのだけど、全メニューの全階層を変更しないといけないので、定数に留めた。
'  │・調整幅を設けるために、マウスホイールと[PageUp][PageDown]キーで別定数にしておいた。
'  └────────────
'┌────┐
'│デザイン├──────────────────────
'└┬───┘
'  │『ホイールと[PageUp][PageDown]キーの違い』
'  │・ホイール … メニューをスクロールする。上下ボタンを押すのと同じ動作。選択位置は変わらない。
'  │・[PageUp][PageDown]キー … メニューをキー操作していることを前提とすると、続けて[↑][↓]キーを使えないといけないので、選択位置自体を動かす。
'  └────────────





''' <summary>
''' 少しスクロールが便利なContextMenuStrip
''' </summary>
Public Class ContextMenuStripEx

    Inherits ContextMenuStrip '基底クラス



    ''' <summary>ホイール1ノッチ毎のスクロール行数</summary>
    Private Const LINES_PER_NOTCH As Integer = 5

    ''' <summary>[PageUp][PageDown]キー1押下毎のスクロール行数</summary>
    Private Const LINES_PER_PAGE As Integer = 5



    ''' <summary>
    ''' 指定アイテムが表示領域内に入るようにスクロールする
    ''' </summary>
    ''' <param name="item">対象アイテム</param>
    Public Sub ScrollIntoView(item As ToolStripItem)

        If item IsNot Nothing AndAlso Not Me.ClientRectangle.Contains(item.Bounds) Then

            'ToolStripDropDownMenu.ChangeSelection(item As ToolStripItem)メソッド
            '指定したアイテムまでスクロールしてくれる。
            '※ あと1行で見えない項目にはスクロールしてくれない…（スクロールボタンの分を勘定に入れ忘れているんじゃない？）。

            '=== メソッドを抽出して叩く ===
            Dim t = GetType(ToolStripDropDownMenu)
            Dim m = t.GetMethod("ChangeSelection", Reflection.BindingFlags.NonPublic Or Reflection.BindingFlags.InvokeMethod Or Reflection.BindingFlags.Instance, Nothing, New Type() {GetType(ToolStripItem)}, Nothing)
            If m IsNot Nothing Then
                m.Invoke(Me, New Object() {item})
            End If

        End If

    End Sub

    ''' <summary>
    ''' スクロールする
    ''' </summary>
    ''' <param name="up">True=上へ False=下へ</param>
    ''' <param name="lines">行数</param>
    Public Sub PerformScroll(up As Boolean, lines As Integer)

        Dim t = GetType(ToolStripDropDownMenu)
        Dim p = t.GetField(If(up, "upScrollButton", "downScrollButton"), Reflection.BindingFlags.NonPublic Or Reflection.BindingFlags.GetField Or Reflection.BindingFlags.Instance)

        If p IsNot Nothing Then

            '上へボタン … ToolStripControlHost.upScrollButton
            '下へボタン … ToolStripControlHost.downScrollButton
            'メソッド   … ToolStripDropDownMenu.ScrollInternal(up As Boolean) → upがTrueなら上へ、Falseなら下へ、1行スクロールする。

            '=== スクロール用のボタンとメソッドを抽出 ===
            Dim b = DirectCast(p.GetValue(Me), ToolStripControlHost)
            Dim m = t.GetMethod("ScrollInternal", Reflection.BindingFlags.NonPublic Or Reflection.BindingFlags.InvokeMethod Or Reflection.BindingFlags.Instance, Nothing, New Type() {GetType(Boolean)}, Nothing)

            '=== スクロール用ボタンが有効ならメソッドを叩く ===
            For i = 0 To lines - 1
                If b IsNot Nothing AndAlso b.Visible AndAlso b.Enabled AndAlso m IsNot Nothing Then
                    m.Invoke(Me, New Object() {up})
                Else
                    Exit For
                End If
            Next

        End If

    End Sub

    Protected Overrides Sub OnMouseWheel(e As MouseEventArgs)

        '=== ホイール操作によるスクロール ===
        PerformScroll(e.Delta > 0, LINES_PER_NOTCH)
        DirectCast(e, HandledMouseEventArgs).Handled = True

        '=== 本来の処理 ===
        MyBase.OnMouseWheel(e)

    End Sub

    Protected Overrides Sub OnKeyUp(e As KeyEventArgs)

        '=== キー操作によるスクロール ===
        Select Case e.KeyCode

            Case Keys.Up, Keys.Down, Keys.Left, Keys.Right, Keys.Apps

                '特別何もしない（アクセスキーと区別するために必要）

            Case Keys.PageUp

                SkipSelectItem(True)
                e.Handled = True

            Case Keys.PageDown

                SkipSelectItem(False)
                e.Handled = True

            Case Else 'それ以外はアクセスキーと見なす

                Dim item As ToolStripItem = Nothing

                For Each i As ToolStripItem In Me.Items
                    If i.Selected Then
                        item = i
                        Exit For
                    End If
                Next
                ScrollIntoView(item)

                e.Handled = IsNothing(item)

        End Select

        '=== 本来の処理 ===
        MyBase.OnKeyUp(e)

    End Sub

    ''' <summary>
    ''' 選択位置をずらす
    ''' </summary>
    ''' <param name="up">ずらす方向（上=True, 下=False）</param>
    Private Sub SkipSelectItem(up As Boolean)

        '=== 現在選択されているアイテムを探す ===
        Dim index = -1 '選択アイテムの番号

        For i = 0 To Me.Items.Count - 1
            If Me.Items(i).Selected Then
                index = i
                Exit For
            End If
        Next

        '=== 選択可能なアイテムを数えて移動先を決める ===
        Dim hop = If(up, -1, 1), peek = index, count = 0 '検索方向, 確認中アイテム番号, 発見数

        Do Until count = LINES_PER_PAGE

            peek += hop

            If peek < 0 OrElse peek = Me.Items.Count Then
                Exit Do
            End If

            Dim check = Me.Items(peek)
            If check.CanSelect AndAlso check.Visible Then
                count += 1
                index = peek
            End If

        Loop

        '=== 移動先アイテムを選択する ===
        If index > -1 Then
            Dim item = Me.Items(index)
            item.Select() 'ToolStripComboBox,ToolStripTextBoxが無反応…？
            ScrollIntoView(item)
        End If

    End Sub

    '''' <summary>
    '''' 表示範囲内にあるアイテムを探す
    '''' </summary>
    '''' <param name="up">検索方向</param>
    '''' <returns>アイテム</returns>
    'Private Function SearchIntoView(up As Boolean) As ToolStripItem

    '    Dim item As ToolStripItem = Nothing

    '    Dim s = If(up, Me.Items.Count - 1, 0),
    '        e = Me.Items.Count - 1 - s,
    '        d = Math.Sign(e - s)

    '    For i = s To e Step d
    '        item = Me.Items(i)
    '        If Me.ClientRectangle.Contains(item.Bounds) AndAlso item.CanSelect Then
    '            'ContextMenuStrip.ClientRectangle.Contains(ToolStripItem.Bounds)で特定のアイテムが画面上に表示されているかを調べられる。
    '            'ただし、ContextMenuStrip.ClientRectangleで得られる範囲は、上下スクロールボタンも含むので注意。実用的にはボタンの高さ分を差し引く必要があり面倒。

    '            Exit For
    '        End If

    '    Next

    '    Return item

    'End Function

End Class
