﻿''' <summary>
''' 経過時間計測コントロール
''' </summary>
Public Class TimeKeeper

    ''' <summary>時間計測</summary>
    Private ReadOnly Watch As New Stopwatch '変数でTimer.Intervalの値を累計した方が軽そうだけど、より正確であろうStopWatchを使った。とは言え、確認間隔がTimer.Intervalなので無意味かも。

    ''' <summary>定期確認</summary>
    Private WithEvents Clock As New Timer



    '============================================================================================



    ''' <summary>
    ''' 定期連絡イベント
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Public Event Periodically(sender As Object, e As EventArgs)

    ''' <summary>
    ''' 作動状況の変化イベント
    ''' </summary>
    ''' <param name="sender"></param>
    Public Event ActuationChanged(sender As Object)



    '============================================================================================



    ''' <summary>
    ''' 現在の動作状況を取得する
    ''' </summary>
    ''' <returns></returns>
    Public ReadOnly Property IsRunning As Boolean

        Get
            Return Watch.IsRunning
        End Get

    End Property

    ''' <summary>
    ''' 監視間隔の設定または取得する
    ''' </summary>
    ''' <returns>ミリ秒</returns>
    Public Property Interval As Integer

        Get
            Return Clock.Interval
        End Get
        Set(value As Integer)
            Clock.Interval = value
        End Set

    End Property

    ''' <summary>
    ''' 現在の経過時間
    ''' </summary>
    ''' <returns>ミリ秒</returns>
    Public ReadOnly Property ElapsedMilliseconds As Long

        Get
            Return Watch.ElapsedMilliseconds
        End Get

    End Property



    '============================================================================================



    ''' <summary>
    ''' 計測を開始する
    ''' </summary>
    Public Sub Start()

        Watch.Restart()
        Clock.Start()

        RaiseEvent ActuationChanged(Me)

    End Sub

    ''' <summary>
    ''' 計測を停止する
    ''' </summary>
    Public Sub [Stop]()

        If IsRunning Then
            Watch.Stop()
            Clock.Stop()

            RaiseEvent ActuationChanged(Me)
        End If

    End Sub


    '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


    Private Sub Clock_Tick(sender As Object, e As EventArgs) Handles Clock.Tick

        '=== 「監視間隔の経過をお知らせします」 ===
        RaiseEvent Periodically(sender, e)

    End Sub

End Class
