﻿'┌────┐
'│技術要点├──────────────────────
'└┬───┘
'  │『タスクトレイアイコンに一定時間マウスカーソルが乗ったのを知る基本的な考え方』
'  │・残念ながら、《NotifyIcon》クラス(タスクトレイアイコン)には、マウスが重なったり外れたりを知るのに便利な《MouseEnter》や《MouseLeave》イベントがありません。
'  │・マウスカーソルがアイコン内に入ったことは、《MouseMove》イベントで把握できる。そこで、イベント発生時のマウスカーソルの位置をスクリーン座標で記憶し、ホバー時間カウント用タイマーが動いていなかったら動かす。
'  │・ホバー時間カウント用タイマーの割り込み（《Tick》イベント）で、マウスカーソルの位置（スクリーン座標）を取得し、先に記憶しておいた座標と比較する。同じならアイコンに乗ったまま、違うなら外れたと判断できる（アイコン内で移動していても、《MouseMove》イベントで座標が更新されるので、タイミングが悪くない限り対応可能で許容範囲内としちゃう）。
'  │
'  │『タスクトレイアイコン(NotifyIcon)の左右クリックで異なるコンテクストメニューを表示する方法（電源オプションのアイコンのような感じ）・・・“内部メソッド呼び出し方式”』（本作での実装と少々異なる部分有り）
'  │・NotifyIconの《MouseDown》イベントで、押されたボタンに対応するメニューを《ContextMenuStrip》プロパティに設定する。
'  │・NotifyIconの《MouseUp》イベントで、右以外のクリックを受け取ったら、NotifyIconの内部メソッドを直接呼び出して右クリック時と同じコンテキストメニュー展開処理を行わせる（《PopupLaunchMenu()》のコード参照）。もちろん、右クリック時は、自動的に処理される。
'  │・ちなみに、NotifyIconの《MouseDown》/《MouseUp》イベントは、キーボードによる操作でも発生する。（[Enter]=ダブルクリック/[Space]=シングルクリック/[Shift]+[F10] or [アプリケーション]キー=右クリック）
'  │・本作での《ntfTrayIcon.ContextMenuStrip》は、左クリック時のみランチメニューで、それ以外では常にシステムメニューになっている。
'  │※既知の問題
'  │・設定ダイアログのポップアップヘルプを表示中に、ランチメニューをホバーで表示すると、タスクバーにフォームが現れてしまうことがある。
'  │・タスクトレイアイコンをクリックせずにコンテキストメニューを開くと、そのメニュー以外をクリックしても閉じられない場合がある。こうなると、キー操作も受け付けない。Webからの情報により、《.Show()》メソッドが駄目なのかと考え、NotifyIconの内部メソッドを直接呼び出す方法に切り替えたが完治せず。また、《GotFocus》イベントが発生するので、フォーカスを得られていないわけでもないみたい。項目を《Select》しても駄目だった。
'  │  ├→ この症状を、『ホバー展開後不全症』と呼称する。
'  │  ├→ 発症条件は不明。とりあえず、スクリーン（画面）のコンテキストメニューを表示している時にホバー展開すると発症するみたい。スタート画面を表示した直後にホバー展開しても発症する。
'  │  └→ 対応策として、『自動撤収』を設けた。
'  │※“《ContextMenuStrip.Show(～)》メソッド方式”との違い
'  │・コンテキストメニューの表示自体は、《ContextMenuStrip.Show(～)》メソッドでも可能。
'  │・ただし、“《ContextMenuStrip.Show(～)》メソッド方式”の利点と欠点を勘案して、“内部メソッド呼び出し方式”を採用している。
'  │  ├→ 利点 ─ 《.Show()》メソッドで開いたコンテキストメニューは、タスクバーを避けて表示される。よって、自動撤収時にアイコンのゲージとランチメニューの重なりを考慮する必要がなくなり、設定ダイアログやシステム設定ファイルの項目を減らせる。
'  │  └→ 欠点 ┬ 《.Show()》メソッドでは、表示したコンテキストメニューがアイコンに重ならないので、右クリックで表示した状態と差異が生まれる。
'  │            └ 《.Show()》メソッドで開いたコンテキストメニューを表示した状態でタスクトレイアイコンを右クリックした場合、表示中のメニューが消えるだけで右クリック用コンテキストメニューが表示されない時がある。（“内部メソッド呼び出し方式”で上手く行っているのは、たまたまかもしれないけど・・・。）
'  │
'  │『ホバー展開が不発に終わる問題への対処』
'  │・もともと、ホバー展開後不全症を解消するために、SendInput APIを使って、マウスのクリックを再現して開くようにしてみた。
'  │  → ランチメニュー（コンテキストメニュー）をクリックせずに開くため、フォーカスが移ってこないことが影響しているのではないかと考えた。
'  │・ところが今度は、ノートンのウィンドウがアクティブな時や、ノートンのタスクトレイアイコンのコンテキストメニューが表示されている時、またタスクマネージャが開いている時に、ホバー展開するとランチメニューが開かない。
'  │・この問題に対処するため、とりあえず、ポップアップ時にタイマを回して、一定時間を経過してもメニューが開いていない場合、失敗したと見なすことにした。
'  │
'  │『ホバー展開の暴発とNotifyIcon.Textの関係について』
'  │・ある時、マウスを動かしていないのに勝手にホバー展開してしまう謎挙動に気が付いた。
'  │  → どうも、タスクトレイに新たなアイコンが加わった時に、NotifyIcon.Textが空（Nullまたは""）だとMouseMoveイベントが発生するみたい。
'  │・自分自身が起動する時も、アイコンをタスクトレイに登録するので、当然暴発する。マウスカーソルがアイコン上にあるかどうかを知れないので、解決は難しい。ゲージが隠れるので無題にできると嬉しいんだけど…。
'  │
'  │『自動撤収機能の仕組み』
'  │・そもそもタスクトレイアイコンは、アプリケーションとは別の独自のフォームで管理されているのだと考えられる。ためしに、アイコンを右クリックした後[Alt]+[F4]を押すと、アプリケーション本体は終了せず（タスク マネージャで確認）タスクトレイアイコンだけが消えることからも窺える。
'  │・そのため、タスクトレイアイコンをクリックせずにメニューを開くと、隠れフォームにフォーカスが上手く移動せず、ホバー展開後不全症を発症することになる。
'  │・ホバー展開後不全症をどうしても根治できず、困り果てた末に繰り出した最終奥義が「自動撤収」機能である。
'  │○「自動撤収」は、コンテキストメニュー上からマウスカーソルが出たらカウントを始め、一定時間が経過すると強制的に閉じる。カウント中に再びマウスカーソルが入ると、カウントを停止する。
'  │├→ 手法1. 《ToolStripDropDown》または《ToolStripMenuItem.DropDown》が持っている《MouseEnter》と《MouseLeave》イベントを使って、入出を把握する。ところが、上下スクロールボタンにマウスカーソルを乗せると自動撤種が再開し、スクロール操作中に閉じられてしまう事態が判明。
'  ││          ※ どうやら、マウスカーソル関係イベントは、スクロールボタンをメニューの構成物として扱ってくれないみたい。《MouseMove》も駄目だった。とんだ落とし穴だ。
'  │├→ 手法2. 「そもそも、メニュー選択に掛かる時間を含めて自動撤収時間を指定すれば停止する必要はないのではないか？」と試作したが、メニュー選択が忙しない…閉じて欲しい時にまどろっこしい…気もし良くない。
'  │├→ 手法3. 《ToolStripDropDown.MouseEnter》イベントで操作するメニューの描画領域を取得《ContextMenuStrip.RectangleToScreen(ContextMenuStrip.ClientRectangle)》し、タイマ割り込みでマウスカーソル座標と比較する方法を考えた。期待通りスクロールボタンを含めた領域を返してくれた。
'  ││          ※ ただし、《ToolStripDropDown.MouseEnter》イベントを使用するため、メニュー階層間を渡る際に、一旦メニュー外に出てから、直接スクロールボタンに乗ると、反応できない。この状況がどれ程の頻度で発生するかわからないが…。
'  ││          → 入出確認間隔は、100msで過不足ない（調整する意味がない）感じだったので、パラメータ化しなかった。
'  │└→ 手法4. 手法3を踏襲しながら、開かれた全階層の領域を保持し、マウスカーソル座標が何れかの領域に含まれていたらメニューに乗っていると判断する。メニュー階層が閉じたら、その領域を抹消する。
'  │            ※ 現在の実装は、抹消処理において保持した領域から閉じた領域を探すのは手間が掛かりそうなので、簡易的な（《ToolStripDropDown.Closed》イベントの発生分だけ削除する）手法を採っている。不安なのは、幾つかの《ToolStripDropDown.Closed》イベントが発生している間に《ToolStripDropDown.Opened》イベントが挟まると破綻する点。
'  │ → 単に自動撤収を入り切りしたいだけなのに、ドンドン複雑になっていくのが遣る瀬無い。
'  │ → スクロールボタン側からのアプローチ試案：スクロールボタン自体に《MouseEnter》《MouseLeave》を発生させれば、手法1で入出監視できるはず。しかし、実体が掴めないボタンのイベントを扱う方法がわからない。
'  │
'  │『コンテキストメニューの逐次構築』
'  │・コンテキストメニューに項目が一つもない状態で表示しようとすると、《Opening》イベントは発生するが、そこでいくら項目を追加しようとも実際には表示してくれない。これを回避するためにダミーの項目を追加しておく。
'  │・更に下層を持つ項目の場合は、《DropDownMenuItems》に未構築を示すアイテムを追加して、《Opening》イベントの関連付けをしておく。ドロップダウンメニューを開こうとしたら《Opening》イベントが発生するので、未構築アイテムの有無をチェックして必要なら構築処理を行う。
'  │○フォルダ構造を逐次構築するメリット
'  │  ├ 1. フォルダ構造を読み込むには、「再帰呼び出し」処理が簡単。しかし、再帰回数が深くなりすぎるとエラー（StackOverflowException）になってしまう。そこで、コンテキストメニューの開かれようとしている層を逐次構築することで、このエラーを回避できる。
'  │  ├ 2. 更に、HDDへのアクセス（メニュー構築に掛かる時間）を分散できる。
'  │  └ 3. あるメニューアイテム以下を再構築することも可能となる。
'  │※既知の問題：ドロップダウンメニューの構築中に他のイベント（項目を選択したり、他のドロップダウンメニューを開いたり）が発生した場合、メニューが表示される前に（《Opening》と《Opened》イベントの間）そのイベントが実行され、その後どろぷダウンメニューが表示されることになる。こうなると、親のメニューは消されドロップダウンメニューのみが取り残されたり、複数のドロップダウンメニューが表示されたりしてしまう。
'  │
'  │『ランチメニュー登録フォルダの場所』
'  │・ランチメニュー登録フォルダのパスはローカル且つWindows形式のフルパスでないといけない。
'  │  → 「頭に"file://"が付くURI」, 「頭に"\\コンピュータ名"が付くUNC」, 短縮パスは、登録内容の変更監視トレース処理に支障をきたす。どうしてもネットワーク上に設置したければ、ローカルにマウント（ドライブレターを割り当てる）する。
'  │  → ファイルに関連付けられているアイコンを取得するSystem.Drawing.Icon.ExtractAssociatedIconメソッドが、UNCに対応していない。
'  │  → 変更監視トレースにはフルパスが必要。IO.Path.GetFullPathメソッドにはURIやUNCを渡せない。URIからWindowsパスに変換したり、URI.IsUncメソッドでUNCか調べる溜めに、URIクラスを創ろうとすると、フルパスが必要。フルパスを得るには、IO.Path.GetFullPathメソッドを使う。この無限ループ。
'  │
'  │『ランチメニューの更新』
'  │・《FileSystemWatcher》クラスで、ランチメニュー登録フォルダを監視する。
'  │・変更アイテムがフォルダであれファイルであれ、その親フォルダに相当する格納アイテム以降を全て再構築する。
'  │  └→ 《FileSystemWatcher》クラスのイベントで変化の内容（追加・削除・改名など）を知れるため、変更箇所のみを個別に対処しようと思えばできるかもしれないけど、そこに手を出すとすごく面倒なことになりそうなので断念。それでも、以前はルートから全再構築だったことに比べると、幾分ましになったと思う。
'  │
'  │『「バージョン情報を閉じる」/「本作を終了する」/「OSが終了（シャットダウンまたはログオフ）する」を区別する方法』
'  │・《FormMain》がバージョン情報を兼ねているので、バージョン情報を表示している時に[×]ボタンや[Alt]+[F4]を押されたら困る。
'  │・意図しない終了を回避するには、《Me.FormClosing》イベントでキャンセル《e.Cancel = True》する方法があるが、これをすると、今度はOSの終了が阻害されてしまう。
'  │・OSの終了を察知する方法には、《Microsoft.Win32.SystemEvents.SessionEnding》イベントがある。しかし、《Me.FormClosing》と《Microsoft.Win32.SystemEvents.SessionEnding》は全く別の仕組みであるため、どっちが先に発生するか分からない。よって、この方法では対処不可能である。（要するに、「通常終了」と「ログオフ」を区別したい時意外、《Microsoft.Win32.SystemEvents.SessionEnding》の出番はなさそう・・・。）
'  │○以上のことから下記の点を《Me.FormClosing》で判断して対処する。（そもそも、《Me.FormClosing》で終了理由を知れることを知らなかった。）
'  │⇒「バージョン情報を閉じる」 → 《ntfTrayIcon》が表示されている。
'  │⇒「本作を終了する」         → 《ntfTrayIcon》が表示されていない。（システムメニューの[終了]をクリックした時に消しておく。起動時エラーによる終了の時点では、まだ表示されていない。）
'  │⇒「OSが終了する」           → 《Me.FormClosing》イベントの《e.CloseReason》プロパティの内容で判断する。（MDIやOwnerは存在しないので、「ユーザの操作《CloseReason.UserClosing》」以外をOSによる終了とみなすことにした。ちなみに、「シャットダウン」「再起動」「ログオフ」共に《CloseReason.WindowsShutDown》になる。）
'  │
'  │『設定コンソールと情報をやり取りする方法』
'  │・設定コンソールへの情報提供は、“コマンドライン引数”で渡す。
'  │・設定コンソールからは、“終了コード”で受け取る（設定コンソールの掲示板を参照）。
'  │
'  │『システムメニュー項目の排他利用制限について』
'  │・バージョン情報と裏メニュー     ・・・ バージョン情報が鍵入力ダイアログの親フォームとなるため、バージョン情報を先に表示すると鍵入力を終了するまで閉じられなくなる（逆の場合は閉じられる）など、ややこしくなるので同時利用禁止とする。
'  │・裏メニューと設定コンソール     ・・・ 鍵入力ダイアログを表示後に鍵を変更されると、現在の設定内容と鍵入力ダイアログが持っている鍵で食い違いが発生してややこしいため、同時利用禁止とする。
'  │・バージョン情報と設定コンソール ・・・ 設定コンソールが本体から完全に独立したため、同時利用可能。
'  └────────────
'┌────┐
'│デザイン├──────────────────────
'└┬───┘
'  │『[裏メニュー]に[見る]と[見ない]をわざわざ設けている理由』
'  │・現在の鍵の開閉状況と、行おうとする作業を、はっきり区別し認識してもらうため。現況をアイコンで、作業をメニュー項目で表現する。
'  │・現状を表すアイコンの意味を知らせるための見本として、子項目を使う。
'  └────────────



''' <summary>
''' メイン兼バージョン情報
''' </summary>
''' <remarks></remarks>
Public Class FormMain

    ''' <summary>スタートアップに登録するショートカットのパス</summary>
    Private ReadOnly StartupShortCutPath As String = IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Startup), IO.Path.GetFileNameWithoutExtension(Application.ExecutablePath) & ".lnk")



    '============================================================================================



    ''' <summary>ランチメニュー構築中フラグ</summary>
    Private WithEvents LaunchMenuBuilding As New BooleanWithEvent(False)

    ''' <summary>ランチメニュー一新フラグ</summary>
    Private WithEvents LaunchMenuAllNew As New BooleanWithEvent(False)

    ''' <summary>裏メニュー開錠フラグ</summary>
    Private WithEvents UnLocked As New BooleanWithEvent(True) '初期化で施錠した時にイベントが発生するように、Trueで作成しておく。

    ''' <summary>裏メニュー開閉アニメーションカウンタ</summary>
    Private OpenCloseAnimation As Integer


    ''' <summary>構築画像用ブラシ（0=虎縞, 1=レンガ）</summary>
    Private ReadOnly BuildBrush As Drawing2D.HatchBrush() = {
        New Drawing2D.HatchBrush(Drawing2D.HatchStyle.DarkUpwardDiagonal, Color.Black, Color.FromArgb(&HFFFFBF00)), '&HFFFFBF00 → GoldとOrangeの中間辺り。
        New Drawing2D.HatchBrush(Drawing2D.HatchStyle.HorizontalBrick, Color.OldLace, Color.FromArgb(&HFF9C4836)) '&HFF9C4836 → レンガ色、Color.FireBrickでは赤過ぎるかな。
    }

    ''' <summary>システム設定ファイルを前回読み込んだ時点の最終更新日（0=システム設定, 1=鍵設定）</summary>
    Private ReadOnly SettingFileTimeStamp() As Date = {Nothing, Nothing}

    ''' <summary>[閉じる]メニューにアイコンを表示するオマケ</summary>
    Private IconClone As Boolean = False



    '============================================================================================



    Private Sub FormMain_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load

        '=== 準備 ===
        '--- フォーム（バージョン情報） ---
        With My.Application.Info
            lblTitle.Text = String.Format("{0} {1}{3}{2}", $"{ .Title} {Environment.NewLine}".Split(" "c))
            btnVersion.Text = "Version " & .Version.ToString
            lnkCopyright.Text = .Copyright & Environment.NewLine & .CompanyName
            lnkCopyright.Links.Add(lnkCopyright.Text.IndexOf(.CompanyName), .CompanyName.Length, IO.Path.Combine(.DirectoryPath, "readme.txt"))
            ttpLink.SetToolTip(lnkCopyright, lnkCopyright.Links(0).LinkData)
        End With
        lblArchitecture.Text = $"Process : {If(Environment.Is64BitProcess, "x64", "x86")} (OS : {If(Environment.Is64BitOperatingSystem, "x64", "x86")})"

        '--- ランチメニュー登録フォルダの監視 ---
        With fswRegistryItemWatcher '《FileSystemWatcher》のプロパティ設定関連には、どうもバグがあるようだ。↓
            .EnableRaisingEvents = False '監視が実行されていると、他のプロパティが変更できない。なのに、デザイン画面のプロパティウィンドウで《False》に設定しても、なぜか次回読み込み時には勝手に《Ture》に書き換えられる。どうやらプロパティウィンドウの見た目だけ《Ture》で内部では《False》になっているようなのだが、何かの拍子に《Ture》化してしまうみたい。原因不明。仕方がないのでここで無効化する。
            .NotifyFilter = IO.NotifyFilters.FileName Or IO.NotifyFilters.DirectoryName Or IO.NotifyFilters.Attributes 'プロパティウィンドウでは複数選択できないので、ここで設定する。
            .IncludeSubdirectories = True
        End With

        '--- コマンドライン引数 ---
        For Each com As String In My.Application.CommandLineArgs
            Select Case com.ToLower
                Case "/clone"
                    IconClone = True
            End Select
        Next

        '--- システム設定パラメータの読み込みと反映 ---
        Setup()

        '--- 自動起動 ---
#If DEBUG Then
        tsmAutoBoot.ToolTipText &= vbCrLf & """" & IO.Path.GetFileName(StartupShortCutPath) & """" 'デバッグ時のみ、ショートカット名を表示
#End If

    End Sub

    Private Sub FormMain_Shown(sender As Object, e As System.EventArgs) Handles Me.Shown

        '=== フォームを隠す ===
        Me.Visible = False '《Load》イベントで非表示にしても《Show》で《Visible》プロパティが《True》になるから、ここでやるしかない。

        '=== ランチメニューの構築 ===
        If SystemSettings.LaunchMenu.MenuBuild.StartUpBuild Then LaunchMenuRootBuild()

        '=== タスクトレイにアイコンを表示 ===
        ntfTrayIcon.Visible = True

    End Sub

    Private Sub FormMain_FormClosing(sender As Object, e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing

        '=== バージョン情報フォームの[×]をクリックした？ ===
        If e.CloseReason = CloseReason.UserClosing AndAlso Me.Visible AndAlso ntfTrayIcon.Visible Then
            e.Cancel = True '『終了』をキャンセル
            btnClose.PerformClick() 'バージョン情報フォームを閉じる。
        End If

    End Sub

    Private Sub FormMain_FormClosed(sender As Object, e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed

        '=== 終了（これ要るのかなぁ？） ===
        HoverTime.Stop()
        CloseTime.Stop()
        tmrPopupChecker.Stop()
        fswRegistryItemWatcher.EnableRaisingEvents = False
        ntfTrayIcon.Visible = False

    End Sub

    Private Sub tsmExit_Click(sender As System.Object, e As System.EventArgs) Handles tsmExit.Click

        '=== 終了 ===
        Me.Close()

    End Sub

    Private Sub tsmVersion_Click(sender As System.Object, e As System.EventArgs) Handles tsmVersion.Click

        '=== バージョン情報 ===
        With Me

            If .Visible Then

                '--- 表示済みならアクティブにする ---
                .Activate()

            Else

                '--- 他の項目を選択できないようにする ---
                tsmSecretMenu.Enabled = False

                '--- 表示 ---
                .ShowInTaskbar = True
                .Opacity = 1
                .Visible = True
                '《Visible = False》にすることでタスクバーへも表示されなくなので、《ShowInTaskbar》は初期値《True》のままで良いのだが、《Visible = False》は、《Shown》イベントでしか行えないので、一瞬タスクバーに見えてしまう。
                'そこで、デザイン時に《ShowInTaskbar = False》に設定しておき、バージョン情報を表示する時に《True》にする。後は、フォームの表示状態と連動するので、《ShowInTaskbar = False》に設定しなおす必要はない（けど、念のためやってる）。

            End If

        End With

    End Sub

    Private Sub tsmQuickManual_Click(sender As Object, e As EventArgs) Handles tsmQuickManual.Click

        '=== 簡易説明書 ===
        DocumentOpen(IO.Path.Combine(My.Application.Info.DirectoryPath, "QuickManual.txt"))

    End Sub

    Private Sub tsmSetting_Click(sender As System.Object, e As System.EventArgs) Handles tsmSetting.Click

        '=== 設定コンソール起動 ===
        Dim SettingProcess As New Process
        With SettingProcess
            With .StartInfo
                .FileName = IO.Path.Combine(My.Application.Info.DirectoryPath, "SettingConsole.exe")
                .Arguments = $"-title ""{NickName(True)}""" &
                             If(SettingFileModifiedChecker(), " -modified", "") &
                             If((Control.ModifierKeys And Keys.Control) = Keys.Control, " /lockbreak", "") 'コマンドライン引数（外部で変更された形跡があれば知らせる。[Ctrl]キーが押されていたら錠前破りモードで起動する。）
            End With
            .SynchronizingObject = Me
            .EnableRaisingEvents = True

            AddHandler .Exited, AddressOf SettingConsole_Exited '終了イベント定義

            Try

                .Start()

                '同時に「裏メニューの開閉」を利用できないようにする
                tsmSecretMenu.Enabled = False

            Catch ex As Exception

                MessageBox.Show("設定コンソール（外部アプリケーション）を実行できませんでした。" & vbCrLf & vbCrLf & ex.Message & vbCrLf & vbCrLf & .StartInfo.FileName, My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Error)

            End Try
        End With

    End Sub

    Private Sub SettingConsole_Exited(sender As System.Object, e As System.EventArgs)

        '=== 設定コンソール終了 ===
        Dim code As Integer = DirectCast(sender, Process).ExitCode
        Select Case code '終了コード：-2=パラメータ不整合（エラー） / -1=二重起動（エラー） / 0=[キャンセル] / 1=[OK] / 2=[適用]
            Case 1, 2
                '--- 変更内容の反映 ---
                Setup()
                'ntfTrayIcon.ShowBalloonTip(1000, "設定コンソール", "変更を適用しました。" & code, ToolTipIcon.Info)
        End Select
        Debug.WriteLine($"終了コード:Catch = {code}")

        '=== 「裏メニュー」を利用できるようにする ===
        tsmSecretMenu.Enabled = True

    End Sub

    Private Sub tsmAutoBoot_Click(sender As System.Object, e As System.EventArgs) Handles tsmAutoBoot.Click

        '=== 自動起動 ===
        If DirectCast(sender, ToolStripMenuItem).Checked Then

            '--- 解除 ---
            If MessageBox.Show("Windowsのスタートアップに登録したショートカットアイコンを削除します。" & vbCrLf & "次回から、手動で起動しなければならなくなります。" & vbCrLf & vbCrLf & "対象：" & IO.Path.GetFileName(StartupShortCutPath), My.Application.Info.Title, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation) = DialogResult.OK Then
                StartupDelete()
            End If

        Else

            '--- 登録 ---
            If MessageBox.Show("Windowsのスタートアップにショートカットアイコンを登録します。" & vbCrLf & "Windowsと共に、自動的に起動するようにします。" & vbCrLf & vbCrLf & "対象：" & IO.Path.GetFileName(StartupShortCutPath), My.Application.Info.Title, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation) = DialogResult.OK Then
                StartupRegistry()
            End If

        End If

    End Sub

    Private Sub tsmReBuild_Click(sender As System.Object, e As System.EventArgs) Handles tsmRebuild.Click

        '=== ランチメニューの強制再構築 ===
        LaunchMenuAllNew.Value = True

    End Sub

    Private Sub tsmSecretMenuShow_Click(sender As System.Object, e As System.EventArgs) Handles tsmSecretMenuShow.Click

        '=== 裏メニューを開く ===
        Static keyForm As KeyInputBox.FormKey = Nothing 'フォームを静的変数で用意（フォーム表示中に、再度メニューがクリックされた場合に対処するため）

        If keyForm Is Nothing OrElse keyForm.IsDisposed Then

            '--- 他の項目を選択できないようにする ---
            tsmVersion.Enabled = False
            tsmSetting.Enabled = False

            '--- 鍵認証 ---
            keyForm = New KeyInputBox.FormKey(KeySettings, SystemSettings.SecretMenu.MouseWheel) With {
                .Text = NickName(True) & .Text,
                .Icon = Me.Icon,
                .Location = Cursor.Position - New Size(.Width \ 2, .Height \ 2)
            }
            'マウスカーソルが、中心に来るように表示 → 入力しやすい場所 ＝ マウスカーソルの場所 ≒ タスクトレイアイコンの場所（付近） ＝ スクリーンの端
            '[OK]ボタンの中心をマウスカーソルの位置に持って来たい時は、↓の式に換える。
            '.Location = Windows.Forms.Cursor.Position - SystemInformation.BorderSize - New Size(.btnOK.Left + .btnOK.Width \ 2, SystemInformation.CaptionHeight + .btnOK.Top + .btnOK.Height \ 2)

            With keyForm
                If KeySettings.Exists Then

                    'スクリーンからのはみ出し補正
                    '（《FormKey.ResizeEnd》イベントで処理した方がスッキリするけど、ここでしか必要ないし、ここに記述した方が実行ファイルが小さくなる誘惑に負けた。ちょうどここをいじってる時が88.5KBと89.0KBのせめぎ合いやった。他をいじると簡単に吹っ飛ぶ程度やけど。）
                    Dim limit As New Size(Screen.PrimaryScreen.WorkingArea.Width - .Width, Screen.PrimaryScreen.WorkingArea.Height - .Height) 'フォームを表示できる範囲

                    If .Left < 0 Then '左右限補正
                        .Left = 0
                    ElseIf limit.Width < .Left Then
                        .Left = limit.Width
                    End If

                    If .Top < 0 Then '上下限補正
                        .Top = 0
                    ElseIf limit.Height < .Top Then
                        .Top = limit.Height
                    End If

                    '鍵を入力
                    .ShowDialog(Me)

                Else

                    '未設定
                    .DialogResult = DialogResult.OK

                End If
            End With

            '--- 裏メニューの開示を許可 ---
            If keyForm.DialogResult = DialogResult.OK Then

                'タスクトレイアイコンのアニメーション
                SecretMenuAnimation(1, 250)
                SecretMenuAnimation(2, 500)
                SecretMenuAnimation(0, 0)

                '開錠
                UnLocked.Value = True

            End If

            '--- 後片付け ---
            keyForm.Dispose()
            keyForm = Nothing

            '--- 他の項目を選択できるようにする ---
            tsmVersion.Enabled = True
            tsmSetting.Enabled = True

        Else

            '--- 表示済みならアクティブにする ---
            keyForm.Activate()

        End If

    End Sub

    Private Sub tsmSecretMenuHide_Click(sender As System.Object, e As System.EventArgs) Handles tsmSecretMenuHide.Click

        '=== 裏メニューを閉じる ===
        'タスクトレイアイコンのアニメーション
        SecretMenuAnimation(-2, 250)
        SecretMenuAnimation(-1, 500)
        SecretMenuAnimation(0, 0)

        '施錠
        UnLocked.Value = False

    End Sub

    ''' <summary>
    ''' 裏メニューの開閉アニメーション
    ''' </summary>
    ''' <param name="count">カウント</param>
    ''' <param name="wait">待ち時間(ms)</param>
    Private Sub SecretMenuAnimation(count As Integer, wait As Integer)

        OpenCloseAnimation = count
        ChangeTaskTrayIcon()
        Threading.Thread.Sleep(wait)

    End Sub

    Private Sub picIcon_Paint(sender As Object, e As System.Windows.Forms.PaintEventArgs) Handles picIcon.Paint

        '=== バージョン情報用アイコン描画 ===
        With e.Graphics
            .DrawIcon(Me.Icon, 0, 0) '32*32px
            .DrawIcon(New Icon(Me.Icon, 16, 16), 7, 36) '16*16px（おまけ）
            .DrawImage(Setting.SystemParameter.TaskTrayParameter.IconParameter.GetImage(SystemSettings.TaskTray.Icon.Type), 7, 56)
            .DrawImage(Setting.My.Resources.DWs, 7, picIcon.ClientRectangle.Bottom - 16)
        End With

    End Sub

    Private Sub btnClose_Click(sender As System.Object, e As System.EventArgs) Handles btnClose.Click

        '=== バージョン情報を閉じる ===
        With Me
            .Opacity = 0
            .Visible = False
            .ShowInTaskbar = False '《Visible = False》の後に行わないと、フォームが点滅したようになってしまうみたい。
        End With

        '=== 他の項目を選択できるようにする ===
        tsmSecretMenu.Enabled = True

    End Sub

    Private Sub lnkCompany_LinkClicked(sender As System.Object, e As System.Windows.Forms.LinkLabelLinkClickedEventArgs) Handles lnkCopyright.LinkClicked

        '=== リンクを開く ===
        With e.Link '[e.Link]が、[lnkCompany.Links()]のアイテムを参照しているみたい。
            .Visited = DocumentOpen(.LinkData)
        End With

    End Sub

    Private Sub btnAllVersion_Click(sender As System.Object, e As System.EventArgs) Handles btnVersion.Click

        '=== 関連（してそうな）ファイルのバージョン情報 ===
        Dim spec = ""
        Dim dir = New IO.DirectoryInfo(My.Application.Info.DirectoryPath)
        For Each file In dir.GetFiles("*.exe").Concat(dir.GetFiles("*.dll")) 'exeとdllファイルを取得
            Dim ver = FileVersionInfo.GetVersionInfo(file.FullName) '対象ファイルのバージョン情報を取得
            spec &= String.Format("/{1,-32}{0}Ver.{2}//{3}///", vbTab, $"""{file.Name}""", ver.FileVersion, ver.Comments) '説明文を作成（Tabを挟むと位置が揃うみたい。）
        Next
        spec = spec.Replace("//", vbCrLf).Replace("/", "") '文字列中の必要箇所を改行文字に置換

        MessageBox.Show("本作のフォルダには、以下のEXE,DLLファイルが同居しています。" & New String(vbCrLf, 2) & spec, Me.Text, MessageBoxButtons.OK)

    End Sub

    Private Sub fswRegistryItemWatcher_Changed(sender As System.Object, e As System.IO.FileSystemEventArgs) Handles fswRegistryItemWatcher.Changed, fswRegistryItemWatcher.Created, fswRegistryItemWatcher.Deleted

        '=== ランチメニューに変更有り ===
        LaunchMenuModify(e.FullPath)

    End Sub

    Private Sub fswRegistryItemWatcher_Renamed(sender As System.Object, e As System.IO.RenamedEventArgs) Handles fswRegistryItemWatcher.Renamed

        '=== ランチメニューに変更有り ===
        LaunchMenuModify(e.FullPath)

    End Sub

    Private Sub ctxSysMenu_Opening(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles ctxSystemMenu.Opening

        '=== システムメニューの展開時処理 ===
        If LaunchMenuBuilding.Value Then
            e.Cancel = True '構築中は開かない
        Else
            tsmAutoBoot.Checked = System.IO.File.Exists(StartupShortCutPath) 'スタートアップへの登録済みチェック
        End If

    End Sub

    Private Sub ctxSystemMenu_Opened(sender As Object, e As EventArgs) Handles ctxSystemMenu.Opened

        '=== システムメニューが開いたらホバー監視停止 ===
        HoverTime.Stop()

    End Sub

    Private Sub LaunchMenuAllNew_Changed(sender As Object, e As System.EventArgs) Handles LaunchMenuAllNew.Changed

        With DirectCast(sender, BooleanWithEvent)

            '=== ランチメニューが開いていたら閉じる ===
            If .Value AndAlso ctxLaunchMenu.Visible Then ctxLaunchMenu.Close()

            '=== ランチメニューの一新フラグに変化 ===
            ChangeTaskTrayIcon(.Value)

            '=== ランチメニュー登録フォルダの変更監視の停止 ===
            '（次回再構築するので、変更を監視する意味がない）
            If .Value Then fswRegistryItemWatcher.EnableRaisingEvents = False

        End With

    End Sub

    Private Sub LaunchMenuBuilding_Changed(sender As Object, e As System.EventArgs) Handles LaunchMenuBuilding.Changed

        '=== ランチメニューの更新中フラグに変化 ===
        ChangeTaskTrayIcon()

    End Sub

    Private Sub UnLocked_Changed(sender As Object, e As System.EventArgs) Handles UnLocked.Changed

        '=== 裏メニューの開閉状態が変更された ===
        tsmSecretMenu.Image = If(UnLocked.Value, My.Resources.Open, My.Resources.Close)

        tsmSecretMenuShow.Enabled = Not UnLocked.Value
        tsmSecretMenuHide.Enabled = UnLocked.Value

        LaunchMenuAllNew.Value = True

    End Sub

    ''' <summary>
    ''' タスクトレイ用アイコンを作成して変更する
    ''' </summary>
    ''' <param name="renewal">部材一新フラグ</param>
    ''' <remarks></remarks>
    Private Sub ChangeTaskTrayIcon(Optional renewal As Boolean = False)

        Static iconImage As Bitmap, imageArea As Rectangle
        Static markerArea As Rectangle
        Static gaugeArea As Rectangle, gaugeBrath(1) As SolidBrush

        '=== 部材準備 ===
        '何回も使いそうなのは捨てない（毎回インスタンシングしない分処理が軽くならんかな…実行ファイル容量には響くみたい）。
        If renewal Then
            With SystemSettings.TaskTray
                iconImage = Setting.SystemParameter.TaskTrayParameter.IconParameter.GetImage(.Icon.Type)
                imageArea = Rectangle.Truncate(iconImage.GetBounds(GraphicsUnit.Pixel))
                markerArea = New Rectangle(0, 0, 8, 8) : markerArea.Offset(imageArea.Right - markerArea.Width, imageArea.Bottom - markerArea.Height)
                gaugeArea = imageArea : gaugeArea.Inflate(8, 8) 'アイコン全体をゲージで覆うためにはみ出させる（拡大量は適当）。
                gaugeBrath(0) = New SolidBrush(.HoverPopupper.ActiveColor.Color)
                gaugeBrath(1) = New SolidBrush(.AutoCloser.ActiveColor.Color)
            End With
        End If

        '=== アイコン作成 ===
        Using paper As Bitmap = iconImage.Clone '画用紙

            Using pigment = Graphics.FromImage(paper) '絵の具

                '--- 各種マーク合成 ---
                If LaunchMenuAllNew.Value Then pigment.FillRectangle(BuildBrush(0), markerArea) 'ランチメニュー未構築
                If LaunchMenuBuilding.Value Then pigment.FillRectangle(BuildBrush(1), imageArea) 'ランチメニュー構築中

                '--- ゲージ描画 ---
                Dim b As SolidBrush, t As Single

                If HoverTime.IsRunning Then
                    b = gaugeBrath(0)
                    t = HoverTime.ElapsedMilliseconds / SystemSettings.TaskTray.HoverPopupper.Time * 360
                ElseIf CloseTime.IsRunning Then
                    b = gaugeBrath(1)
                    t = 360 - (CloseTime.ElapsedMilliseconds / SystemSettings.TaskTray.AutoCloser.Time * 360)
                Else
                    b = Nothing
                End If

                If b IsNot Nothing Then
                    pigment.FillPie(b, gaugeArea, -90, t)
                End If

                '--- 裏メニュー開閉アニメーション ---
                If OpenCloseAnimation <> 0 Then
                    pigment.FillRectangle(New SolidBrush(Color.FromArgb(&H80, Color.White)), imageArea)
                    pigment.DrawImage(If(Math.Abs(OpenCloseAnimation) = 1, tsmSecretMenuHide.Image, tsmSecretMenuShow.Image), imageArea) 'ToolStripMenuItem.Imageから持ってくることで、透明色（.ImageTransparentColorプロパティ）が適用された状態で使用できるみたい。
                End If

            End Using

            '--- 画像を反映 ---
            ntfTrayIcon.Icon = Icon.FromHandle(paper.GetHicon) 'BitmapをIconに変換して登録。
            My.Application.DoEvents() 'タスクトレイの更新を行わせる（ホバー継続時間ゲージの満タン状態を見えるようにする）。無くても支障はない。

            If IconClone Then PeculiarlyItem(0).Image = paper.Clone 'メニュー項目は、ハイライト加工される都合上、参照元がなくなるとエラーになるので、クローンにする。

        End Using

    End Sub

    ''' <summary>
    ''' ランチメニュー登録フォルダ存在チェック
    ''' </summary>
    ''' <param name="directoryPath">ランチメニュー登録フォルダのパス</param>
    ''' <returns>存在フラグ</returns>
    ''' <remarks></remarks>
    Public Function LaunchMenuExistenceCheck(directoryPath As String) As Boolean

        '=== フォルダが存在しなければ作成する ===
        Dim ret As Boolean = True '結局、フォルダは有るのか無いのか

        If Not IO.Directory.Exists(directoryPath) Then
            Dim crlf2 = New String(Environment.NewLine, 2)
            Try

                IO.Directory.CreateDirectory(directoryPath)

                Dim msg = "ランチメニュー登録フォルダ作成しました。" & crlf2 & directoryPath
                If ntfTrayIcon.Visible Then
                    ntfTrayIcon.ShowBalloonTip(10000, My.Application.Info.Title, msg, ToolTipIcon.Info)
                Else
                    MessageBox.Show(msg, My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Information)
                End If

            Catch ex As Exception

                MessageBox.Show(String.Format("ランチメニュー登録フォルダの作成に、失敗しました。{0}『{1}』{0}{2}", crlf2, ex.Message, directoryPath),
                                My.Application.Info.Title,
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error)
                ret = False

            End Try
        End If

        Return ret

    End Function

    ''' <summary>
    ''' 設定ファイルの変化を確認
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function SettingFileModifiedChecker() As Boolean

        Return SettingFileTimeStamp(0) <> IO.File.GetLastWriteTime(SystemSettingsFileName) OrElse SettingFileTimeStamp(1) <> IO.File.GetLastWriteTime(keySettingsFileName) '設定ファイルが更新・作成・削除されているか？（ファイルが存在しない時は"1601/01/01 9:00:00"が返されるようので、存在確認をする必要はない。）

    End Function

    ''' <summary>
    ''' 設定ファイルの適用
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub Setup()

        '=== 設定パラメータの読み込み ===
        Dim errorFlag As Boolean = False 'エラーフラグ

        '--- システム設定 ---
        Dim errorPoint As String = ""
        Try
            errorPoint = SettingsLoad(SystemSettingsFileName, SystemSettings)
        Catch ex As Exception
            MessageBox.Show("設定の読み込み・復元中に例外が発生しました。" & vbCrLf & "規定値を使用して実行を継続するので、設定をやり直してください。" & vbCrLf & vbCrLf & ex.Message,
                            My.Application.Info.Title,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error) '読み込み中にエラーが発生した場合は、読み込めた部分も含め何も反映されないようなので、再初期化は不要。
            errorFlag = True
        End Try

        '整合性エラーの警告
        If errorPoint <> "" Then
            MessageBox.Show("システム設定ファイルの記述に致命的な不整合があるため、起動を中止します。" & vbCrLf & "下記の項目に関する設定値を、書き直してください。" & vbCrLf & vbCrLf & errorPoint,
                            My.Application.Info.Title,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error)

            tsmExit.PerformClick() '強制終了※※※※
            Exit Sub
        End If

        'ランチメニュー登録フォルダのパス検証
        Dim msg = Setting.SystemParameter.LaunchMenuParameter.JustifyPathCheck(SystemSettings.LaunchMenu.RegistryPath)
        If Not String.IsNullOrEmpty(msg) Then
            MessageBox.Show(Me, $"{msg}{New String(Environment.NewLine, 2)}起動を中止します。", Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
            'SystemSettings.LaunchMenu.RegistryPath = Setting.SystemParameter.LaunchMenuParameter.DefaultPath '既定のパスで起動する。
            tsmExit.PerformClick() '強制終了※※※※
            Exit Sub
        End If

        '--- 鍵 ---
        Try
            SettingsLoad(keySettingsFileName, KeySettings)
        Catch ex As Exception
            MessageBox.Show("鍵の読み込み・復元中に例外が発生しました。" & vbCrLf & "解除された状態になるので、早急に設定をやり直してください。" & vbCrLf & vbCrLf & ex.Message,
                            My.Application.Info.Title,
                            MessageBoxButtons.OK,
                            MessageBoxIcon.Error) '読み込み中にエラーが発生した場合は、読み込めた部分も含め何も反映されないようなので、再初期化は不要。
            errorFlag = True
        End Try

        '--- 設定ファイルの最終更新日を記憶 ---
        SettingFileTimeStamp(0) = IO.File.GetLastWriteTime(SystemSettingsFileName) 'ファイルが存在しない時は"1601/01/01 9:00:00"が返されるので、ファイルの存在確認はしなくても良い。
        SettingFileTimeStamp(1) = IO.File.GetLastWriteTime(keySettingsFileName)

        '=== 設定パラメータの反映 ===
        With SystemSettings

            '--- タスクトレイアイコン ---
            With .TaskTray
                HoverTime.Interval = .HoverPopupper.AdjustStep
                CloseTime.Interval = .AutoCloser.AdjustStep
            End With

            '--- ランチメニュー ---
            With .LaunchMenu
                LaunchMenuExistenceCheck(.RegistryPath) 'ランチメニューを起動時に読み込まない設定の時でも、起動と同時に登録フォルダが作成されるようにここで実行しておく。
                ctxSystemMenu.Tag = .RegistryPath 'システムメニューでランチメニュー登録フォルダを開く用にパスと入れておく。
                fswRegistryItemWatcher.Path = .RegistryPath

                tmrPopupChecker.Interval = .Popup.PopupCheckTime

                With .Classify
                    If .ColorCoded Then
                        ItemBackColor(0) = .DirectoryItemColor.Color
                        ItemBackColor(1) = .FileItemColor.Color
                    Else
                        ItemBackColor(0) = SystemColors.Menu
                        ItemBackColor(1) = SystemColors.Menu
                    End If
                End With
            End With

            '--- 裏メニュー ---
            tsmSecretMenu.Visible = .SecretMenu.MenuItemVisible

        End With

        '=== 部品準備 ===
        With SystemSettings.LaunchMenu

            '--- 格納アイテム用画像 ---
            '全ての格納アイテムで一つの画像を参照すれば、メモリを節約できるのではないか。
            DirectoryItemImage(0) = New Bitmap(1, 1) 'ダミー。サイズはどうでも良い。
            DirectoryItemImage(2) = New Bitmap(.IconImageScale, .IconImageScale)
            Using grp = Graphics.FromImage(DirectoryItemImage(2))
                Dim r = Rectangle.Truncate(DirectoryItemImage(2).GetBounds(GraphicsUnit.Pixel))

                '未構築印
                Dim a = New Rectangle(0, 0, 8, 16) : a.Offset(r.Right - a.Width, (r.Bottom - a.Height) \ 2)
                grp.FillRectangle(BuildBrush(0), a)
                DirectoryItemImage(1) = DirectoryItemImage(2).Clone

                '構築中印
                grp.FillRectangle(BuildBrush(1), r)
                grp.DrawRectangle(New Pen(Color.FromArgb(&H7F, Color.White)), r)
                r.Offset(-1, -1)
                grp.DrawRectangle(New Pen(Color.FromArgb(&H7F, Color.Black)), r)

                'r.Inflate(0, -r.Height \ 4)
                'grp.FillRectangle(BuildBrush(1), r)
                'r.Inflate(-1, -1) : r.Width -= 1 : r.Height -= 1
                'Dim p = {New Point(r.Left, r.Top), New Point(r.Right, r.Top), New Point(r.Right, r.Bottom), New Point(r.Left, r.Bottom)}
                'grp.DrawLines(New Pen(Color.FromArgb(&H7F, Color.Black), 3), {p(1), p(2), p(3)})
                'grp.DrawLines(New Pen(Color.FromArgb(&H7F, Color.White), 3), {p(3), p(0), p(1)})
            End Using
            'Clipboard.SetImage(DirectoryItemImage(2)) '画像確認

            '--- その他 ---
            NewMenuStrip(.RegistryPath, ctxLaunchMenu)

        End With

        ntfTrayIcon.Text = NickName()
        LaunchMenuAllNew.Value = True 'この先でタスクトレイアイコンが作成される。

        UnLocked.Value = False

    End Sub

    ''' <summary>
    ''' 表題を取得する
    ''' </summary>
    ''' <param name="short">短縮名を得る</param>
    Private Function NickName(Optional [short] As Boolean = False) As String

        With SystemSettings.TaskTray.Icon
            If String.IsNullOrWhiteSpace(.Title) Then
                Dim rx As New Text.RegularExpressions.Regex("(?<=『).*?(?=』)") 'アプリ名“Launch Time 『～』”の～部分にマッチする正規表現
                If String.IsNullOrEmpty(.Title) Then
                    '空文字…アプリ名
                    Return If([short], rx.Match(My.Application.Info.Title).Value, lblTitle.Text)
                Else
                    '空白文字…アイコン名
                    Dim n = Setting.SystemParameter.TaskTrayParameter.IconParameter.GetName(.Type) & "饂飩"
                    Return If([short], n, rx.Replace(lblTitle.Text, n))
                End If
            Else
                'ユーザー設定
                Return .Title
            End If
        End With

    End Function

    ''' <summary>
    ''' ランチメニューに更新があった
    ''' </summary>
    ''' <param name="path">更新検知フォルダのパス</param>
    ''' <remarks></remarks>
    Private Sub LaunchMenuModify(path As String)

        'ランチメニューが開いていたら閉じる
        If ctxLaunchMenu.Visible Then ctxLaunchMenu.Close()

        '一新フラグが既に立っていたら抜ける
        If LaunchMenuAllNew.Value Then Exit Sub '※※※ フラグが立った時点で監視を停止しているのでたいした意味ないけど、何かの時のために。

        '=== 目的のメニューアイテムまでフォルダ名で辿る ===
        Dim errorFlag As Boolean = False 'エラー フラグ
        Dim strip As ContextMenuStripEx = ctxLaunchMenu, '検索対象のメニュー層
            item As ToolStripMenuItem = Nothing '辿ったメニューアイテム

        For Each element As String In IO.Path.GetDirectoryName(path).Replace(fswRegistryItemWatcher.Path, "").Split({IO.Path.DirectorySeparatorChar}, StringSplitOptions.RemoveEmptyEntries)
            '                         1~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 2~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 3~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
            '                         1.更新検知フォルダの親を取得して、2.ルート部分を除去して、3.各フォルダ名を分割（空白は無視）。

            '--- フォルダに対応する格納アイテムを取得 ---
            item = strip.Items(element)
            If item Is Nothing Then
                errorFlag = True
                Exit For '※※ 格納アイテムが見つからないのはエラー
            End If

            '--- 次の層へ ---
            strip = item.DropDown
            If strip.Items(0).Tag = UNREACHED_MARK Then Exit Sub '※※※ 未踏格納アイテムに行きついたら改める必要がない

        Next

        '=== 未踏格納アイテム化 ===
        If item Is Nothing Then
            LaunchMenuAllNew.Value = True 'エラーまたはトレース結果が《Nothing》ならルートのアイテムに変化があったことを示す → 一新フラグを立てる。（エラーメッセージを表示する前に再構築フラグを立てておかないと、メッセージを表示したまま旧メニューを操作できてしまう。）
            If errorFlag Then MessageBox.Show($"変更箇所をランチメニューから検出できませんでした。{Environment.NewLine}ランチメニュー登録フォルダとランチメニューの構成に齟齬がないか確認してください。", My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Error)
        Else
            SetUnreachedTag(item) '未踏情報を書き込む
        End If

    End Sub

    ''' <summary>
    ''' スタートアップに登録
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub StartupRegistry()

        If Not System.IO.Directory.Exists(System.IO.Path.GetDirectoryName(StartupShortCutPath)) Then

            MessageBox.Show("スタートアップのフォルダがありません。" & vbCrLf & "設定をやり直してみるか、Windowsの設定を見直してください。" & vbCrLf & vbCrLf & System.IO.Path.GetDirectoryName(StartupShortCutPath), My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Error)
            'ショートカット作成時にどんなエラーが発生するのかわからないので、念のためスタートアップフォルダの存在チェックも入れてみた。
            'スタートアップのパスをシステムメニュー表示時に取得しているので、確定前にスタートアップの場所が変更されると、困ったことのなるのだろうか？

        ElseIf System.IO.File.Exists(StartupShortCutPath) Then

            MessageBox.Show("既にスタートアップに存在しています。", My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Information)

        Else

            '=== ショートカット作成 ===
            Try

                Dim oShell As Object = CreateObject("WScript.Shell")
                Dim oShortcut As Object = oShell.CreateShortcut(StartupShortCutPath) 'ショートカットのファイル名

                With oShortcut
                    .TargetPath = Application.ExecutablePath 'リンク先のファイル名
                    .WorkingDirectory = Application.StartupPath
                    .Description = My.Application.Info.Description 'コメント
                    '.Arguments = "/a /b /c" 'コマンドパラメータ（"リンク先"の後ろに付く）
                    '.Hotkey = "Ctrl+Alt+Shift+F12" 'ショートカットキー（ホットキー）
                    '.WindowStyle = 1 '実行時の大きさ（1=通常, 3=最大化, 7=最小化）
                    '.IconLocation = Application.ExecutablePath + ",0" 'アイコンのパス（自分のEXEファイルのインデックス0のアイコン）
                    '↑色々設定できるけど、今回は不要なのでコメントアウト！！

                    .Save() '作成
                End With

                '参考
                'DOBON.NET > プログラミング道 > .NET Tips > ファイル、フォルダ > ショートカットを作成する ・・・ http://dobon.net/vb/dotnet/file/createshortcut.html
                'Visual Basic 中学校 > VB.NET サンプル > ショートカットを作成する                         ・・・ http://homepage1.nifty.com/rucio/main/dotnet/Samples/SmpleCnt.htm

            Catch ex As Exception

                MessageBox.Show("スタートアップへの登録に失敗しました。" & vbCrLf & vbCrLf & ex.Message, My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Error)

            End Try

        End If

    End Sub

    ''' <summary>
    ''' スタートアップから削除
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub StartupDelete()

        If Not System.IO.File.Exists(StartupShortCutPath) Then

            MessageBox.Show("既にスタートアップに存在していません。", My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Information)

        Else

            '=== ショートカット削除 ===
            Try

                My.Computer.FileSystem.DeleteFile(StartupShortCutPath, FileIO.UIOption.OnlyErrorDialogs, FileIO.RecycleOption.DeletePermanently)

            Catch ex As Exception

                MessageBox.Show("スタートアップからの削除に失敗しました。" & vbCrLf & vbCrLf & ex.Message, My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Error)

            End Try

        End If

    End Sub

    ''' <summary>
    ''' ドキュメントを開く
    ''' </summary>
    ''' <param name="name">対象ドキュメント</param>
    ''' <returns></returns>
    Private Function DocumentOpen(name As String) As Boolean

        Try
            Process.Start(name)
            Return True
        Catch ex As Exception
            Dim crlf2 As String = Environment.NewLine & Environment.NewLine
            MessageBox.Show("指定先の展開に失敗しました。" & crlf2 & name & crlf2 & ex.Message, My.Application.Info.Title, MessageBoxButtons.OK, MessageBoxIcon.Error)
            Return False
        End Try

    End Function

End Class
