﻿''' <summary>
''' システム設定パラメータ
''' </summary>
''' <remarks></remarks>
<System.Xml.Serialization.XmlRoot("LaunchTimeシステム設定パラメータ")> _
Public Class SystemParameter

    ''' <summary>現在のリビジョン</summary>
    Public Shared ReadOnly NOW_REVISION As Integer = 1

    ''' <summary>リビジョン</summary>
    <Xml.Serialization.XmlAttribute>
    Public Revision As Integer = 0 '※インスタンス時には最新番号の代入が必須。初期値が0なのは、記載されていないファイルをRev.0として扱うため。

    ''' <summary>
    ''' タスクトレイに関する設定
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("タスクトレイ")> _
    Public Property TaskTray As New TaskTrayParameter

    ''' <summary>
    ''' ランチメニューに関する設定
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("ランチメニュー")> _
    Public Property LaunchMenu As New LaunchMenuParameter

    ''' <summary>
    ''' 裏メニューに関する設定
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <System.Xml.Serialization.XmlElement("裏メニュー")> _
    Public Property SecretMenu As New SecretMenuParameter



    '============================================================================================================================



    ''' <summary>
    ''' タスクトレイに関する設定値
    ''' </summary>
    ''' <remarks></remarks>
    Public Class TaskTrayParameter

        ''' <summary>
        ''' アイコンに関する設定
        ''' </summary>
        ''' <returns></returns>
        <System.Xml.Serialization.XmlElement("アイコン")>
        Public Property Icon As New IconParameter

        ''' <summary>
        ''' ホバー展開に関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("ホバー展開")>
        Public Property HoverPopupper As New TimeMeasureParameter With {.AdjustStep = 50, .Time = 1000, .ActiveColor = New SerializableColor With {.Color = Color.FromArgb(&HBF, Color.SpringGreen)}}

        ''' <summary>
        ''' 自動撤収に関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("自動撤収")>
        Public Property AutoCloser As New TimeMeasureParameter With {.AdjustStep = 50, .Time = 5000, .ActiveColor = New SerializableColor With {.Color = Color.FromArgb(&HBF, Color.YellowGreen)}}


        '----------------------------------------------------------------------------------------------------------------------------


        ''' <summary>
        ''' アイコン
        ''' </summary>
        Public Class IconParameter

            ''' <summary>
            ''' 種類
            ''' </summary>
            Public Enum Types As Byte

                ''' <summary>立ち上り</summary>
                Steam = 1

                ''' <summary>立ち上げ</summary>
                Rise

                ''' <summary>持ち上げ</summary>
                Macho

                ''' <summary>打ち上げ</summary>
                Rocket

            End Enum

            ''' <summary>
            ''' 表題の最大文字数（.NET Framework 4.8のNotifyIcon.Textには、63文字制限がある。）
            ''' </summary>
            Public Shared ReadOnly TitleMaxLength As Integer = 63


            '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


            ''' <summary>
            ''' 種類
            ''' </summary>
            ''' <returns></returns>
            <System.Xml.Serialization.XmlElement("種類")>
            Public Property Type As Types = Types.Steam

            ''' <summary>
            ''' 表題
            ''' </summary>
            ''' <returns></returns>
            <System.Xml.Serialization.XmlAttribute("表題")>
            Public Property Title As String = " " '"第1発射台" '"Launch Pad #1"
            '※空白文字を扱いたいので、XMLテキストではなくXML属性として記録する。


            '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


            ''' <summary>
            ''' アイコンの名称を得る
            ''' </summary>
            ''' <param name="type">アイコンの種類</param>
            ''' <returns>名称</returns>
            Public Shared Function GetName(type As Types) As String

                Return Choose(type, "立ち上り", "立ち上げ", "持ち上げ", "打ち上げ")

            End Function

            ''' <summary>
            ''' アイコンの画像を得る
            ''' </summary>
            ''' <param name="type">アイコンの種類</param>
            ''' <returns>画像</returns>
            Public Shared Function GetImage(type As Types) As Bitmap

                Return Choose(type, My.Resources.Steam, My.Resources.Rize, My.Resources.Macho, My.Resources.Rocket)
                '外部からリソースに直接アクセスする方法 … プロジェクト名.My.Resources.リソース名

            End Function

        End Class


        '----------------------------------------------------------------------------------------------------------------------------


        ''' <summary>
        ''' 時間経過計測に関する設定値（コンストラクタを記述したクラスをシリアル化するとエラーになるから、宣言時に初期値設定するしかない。）
        ''' </summary>
        ''' <remarks></remarks>
        Public Class TimeMeasureParameter

            ''' <summary>
            ''' 最長時間（60秒）
            ''' </summary>
            Public Shared ReadOnly AdjustMax As Integer = 60000


            '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


            ''' <summary>
            ''' 計測間隔
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("調整間隔")>
            Public Property AdjustStep As Integer = 50

            ''' <summary>
            ''' 時間
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("時間")>
            Public Property Time As Integer = 2500

            ''' <summary>
            ''' 作動色
            ''' </summary>
            ''' <returns></returns>
            <System.Xml.Serialization.XmlElement("作動色")>
            Public Property ActiveColor As New SerializableColor With {.Color = Color.FromArgb(&HFF * 0.75, Color.Orange)}

            ''' <summary>
            ''' 有効かどうかを知る。
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            Public ReadOnly Property Enabled As Boolean

                Get
                    Return (Time > 0)
                End Get

            End Property

        End Class

    End Class



    '============================================================================================================================



    ''' <summary>
    ''' ランチメニューに関する設定値
    ''' </summary>
    ''' <remarks></remarks>
    Public Class LaunchMenuParameter

        ''' <summary>小さいアイコンの画像寸</summary>
        Public Shared ReadOnly SmallIconScale As Byte = 16

        ''' <summary>大きいアイコンの画像寸</summary>
        Public Shared ReadOnly LargeIconScale As Byte = 32

        ''' <summary>アイコンの最大画像寸（あまり大きいとコンテクストメニューのスクロールで不具合を起こすので、64px以下が無難。）</summary>
        Public Shared ReadOnly MaximumIconScale As Byte = 64


        '----------------------------------------------------------------------------------------------------------------------------


        ''' <summary>
        ''' ランチメニュー登録フォルダの基点パス
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("ランチメニュー登録フォルダ")>
        Public Property RegistryPath As String = IO.Path.Combine(My.Application.Info.DirectoryPath, "LaunchMenu")

        ''' <summary>
        ''' 構築に関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("構築")>
        Public Property MenuBuild As New MenuBuildParameter

        ''' <summary>
        ''' 展開に関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("展開")>
        Public Property Popup As New PopupParameter

        ''' <summary>
        ''' 手動撤収項目の常設化
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("手動撤収")>
        Public Property HandClose As Boolean = True
        '※ 手動展開で[閉じる]項目を“表示する”理由 …
        '  ├ スタート画面を開閉した直後だと、手動展開でも閉じられなくなるため。（タスクバーにアイコンが現れるので、これをクリックすれば閉じられる。）
        '  └ 展開方法によって表示項目数の変化を小さくするため。（感覚的に「上から○番目」という風に選択できるように。）
        '※ ホバー展開で[閉じる]項目を“表示しない”理由 …
        '  └ 自動撤収機能によりメニューに閉じる手段がなくても何とかなる場面が増えたから。なるべく無駄な要素を減らせるように。

        ''' <summary>
        ''' 一括実行の許可
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("一括実行")>
        Public Property BatchRun As Boolean = True

        ''' <summary>
        ''' ランチメニュー登録フォルダのサブフォルダを開けるようにする
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("登録サブフォルダ展開")>
        Public Property SubDirectoryOpener As Boolean = True

        ''' <summary>
        ''' ソート
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("ソート")>
        Public Property Sort As New SortParameter

        ''' <summary>
        ''' 分類
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("分類")>
        Public Property Classify As New ClassifyParameter

        ''' <summary>
        ''' アイコン画像の大きさ（0はエラーになる。あまりに大きいとメニューのスクロール処理で不具合を起こす。16～64が妥当。）
        ''' </summary>
        ''' <returns></returns>
        <System.Xml.Serialization.XmlElement("アイコン画像寸")>
        Public Property IconImageScale As Byte = SmallIconScale

        ''' <summary>
        ''' 画像を拡大縮小する手法
        ''' </summary>
        ''' <returns></returns>
        <System.Xml.Serialization.XmlElement("画像拡縮手法")>
        Public Property InterpolationMode As Drawing2D.InterpolationMode = Drawing2D.InterpolationMode.HighQualityBicubic

        ''' <summary>
        ''' アイテムの登録名に拡張子を表示
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("拡張子表示")>
        Public Property ShowExtension As Boolean = False

        ''' <summary>
        ''' 拡張コマンドライン引数モードの有効化
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("拡張コマンドライン引数")>
        Public Property ExCommandLineOption As Boolean = False

        ''' <summary>
        ''' ランチメニュー登録フォルダのパスの正当性を検証する
        ''' </summary>
        ''' <param name="path">フォルダのパス</param>
        ''' <returns>エラー文（問題がなければNothing）</returns>
        Public Shared Function JustifyPathCheck(ByRef path As String) As String

            Try

                '=== 前後空白と末尾のディレクトリ区切り文字("\")を削除 ===
                'ランチメニューの登録内容変更箇所のトレース時に親フォルダを取得するのに邪魔なので取っておく。
                path = path.Trim.TrimEnd(IO.Path.DirectorySeparatorChar).TrimEnd

                '=== 空文字・不正文字混入を検出 ===
                If String.IsNullOrWhiteSpace(path) OrElse path.IndexOfAny(IO.Path.GetInvalidPathChars) > -1 Then
                    Return "ランチメニュー登録フォルダのパスが不正です。"
                End If

                '=== 短縮パスをフルパスに戻す ===
                path = IO.Path.GetFullPath(path) '無用に重複した区切り文字("～\\～")も削除してくれる。

                '=== NUCを排除 ===
                'ファイルに関連付けられているアイコンを取得するSystem.Drawing.Icon.ExtractAssociatedIconメソッドが、UNCに対応していない。
                'System.IO.FileSystemWatcherもNUCに無反応そう。
                If New Uri(path).IsUnc Then
                    Return $"ランチメニュー登録フォルダのパスは、UNCに対応していません。{Environment.NewLine}ネットワークに配置したい場合は、ローカルにマウントする必要があります。"
                End If

                '※ URIは、IO.Path.GetFullPathメソッドでエラーになるので排除される。
                '   そもそも、Uri.IsFileメソッドでURIパスか判別できるけど、Uriクラスのインスタンシングにフルパスが必要なのに、IO.Path.GetFullPathメソッドがURIパスを受け付けないので、対処のしようがない。
                '   普通にWindowsを使っている中で、ローカルドライブのパスをURIで指定することはほぼ無いだろうから、無視することにした。
                '   （ちなみに、u = New Uri(パス) : u.LocalPath + Uri.UnescapeDataString(u.Fragment)でURIからWindows形式に変換できる。）

            Catch ex As Exception

                Return $"ランチメニュー登録フォルダのパスで問題が発生しました。{New String(Environment.NewLine, 2)}{ex.Message}"

            End Try

            Return Nothing

        End Function


        '----------------------------------------------------------------------------------------------------------------------------


        ''' <summary>
        ''' メニュー構築に関する設定値
        ''' </summary>
        ''' <remarks></remarks>
        Public Class MenuBuildParameter

            ''' <summary>
            ''' 再帰呼び出しの限界深度（[StackOverflowException]回避用、10層も潜れば十分実用に耐える？）
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("再帰呼出限界深度")>
            Public Property RecursiveCallMax As Integer = 10

            ''' <summary>
            ''' 一括で構築する深度
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("一括構築深度")>
            Public Property Depth As Integer = 0

            ''' <summary>
            ''' 起動時にメニューを構築する
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("起動時構築")>
            Public Property StartUpBuild As Boolean = False

        End Class


        '----------------------------------------------------------------------------------------------------------------------------


        ''' <summary>
        ''' 展開に関する設定値
        ''' </summary>
        Public Class PopupParameter

            ''' <summary>
            ''' 展開手法
            ''' </summary>
            Public Enum WayType As Byte

                ''' <summary>メソッド呼出</summary>
                CallMethod

                ''' <summary>メソッド抽出</summary>
                DirectMethod

                ''' <summary>右クリック再現</summary>
                RightClick

            End Enum


            '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


            ''' <summary>
            ''' ホバーによる展開手段
            ''' </summary>
            ''' <returns></returns>
            <System.Xml.Serialization.XmlElement("ホバー")>
            Public Property Hover As WayType = WayType.RightClick

            ''' <summary>
            ''' 左クリックによる展開手段
            ''' </summary>
            ''' <returns></returns>
            <System.Xml.Serialization.XmlElement("左クリック")>
            Public Property Click As WayType = WayType.DirectMethod

            ''' <summary>
            ''' 再試行による展開手段
            ''' </summary>
            ''' <returns></returns>
            <System.Xml.Serialization.XmlElement("再試行")>
            Public Property Retry As WayType = WayType.DirectMethod '『タスクバーを隠す』モードになっている場合も考えられるのでメソッド抽出が無難。

            ''' <summary>
            ''' 展開の判定成否を確認するまでの時間
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("成否確認時間")>
            Public Property PopupCheckTime As Integer = 1000

        End Class


        '----------------------------------------------------------------------------------------------------------------------------


        ''' <summary>
        ''' ソートに関する設定値
        ''' </summary>
        Public Class SortParameter

            '※ OrderType と DirectionID を一まとめにできるけど、そうすと実行ファイルが肥大化してしまったので止めた。
            '<Flags>
            'Public Enum OrderTypes As Byte

            '    '価が偶数（第0ビットが降りている）なら昇順、奇数（第0ビットが立っている）なら降順。
            '    'Order And Not Reverse で逆順にするか判定できる（第0ビットを取り出せる）。

            '    ''' <summary>何もしない</summary>
            '    DoNothing = &B0

            '    ''' <summary>逆順</summary>
            '    Reverse = &B1

            '    ''' <summary>更新日時順</summary>
            '    LastWriteTime = &B1000

            '    ''' <summary>作成日時順</summary>
            '    CreationTime = &B110

            '    ''' <summary>自然順（名前）</summary>
            '    Natural = &B100

            '    ''' <summary>辞書順（名前）</summary>
            '    Dictionary = &B10 '(Lexicographical Order)

            '    '列挙型をシリアライズする時、同じ値なら先に宣言した要素が優先される。
            '    '今回の値を数値の小さい方から並べると、"CreationTime"が"Dictionary Natural"になってしまう。
            '    '<Flags>を付けたのは、各値に"Reverse"の第0ビットが立っているかを知りたいからなので、他と値が被る要素が出てくる。
            '    'なので、被ってしまう値を先に宣言して優先させるのが重要になる。

            'End Enum

            ''' <summary>
            ''' 順序
            ''' </summary>
            Public Enum OrderTypes As Byte

                ''' <summary>辞書順（名前）</summary>
                Dictionary '(Lexicographical Order)

                ''' <summary>自然順（名前）</summary>
                Natural

                ''' <summary>作成日時順</summary>
                CreationTime

                ''' <summary>更新日時順</summary>
                LastWriteTime

            End Enum

            ''' <summary>
            ''' 方向
            ''' </summary>
            Public Enum Directions As Byte

                ''' <summary>昇順</summary>
                Ascending

                ''' <summary>降順</summary>
                Descending

            End Enum


            '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


            ''' <summary>
            ''' 順序
            ''' </summary>
            ''' <returns></returns>
            <Xml.Serialization.XmlElement("順序")>
            Public Property Order As OrderTypes = OrderTypes.Natural

            ''' <summary>
            ''' 方向
            ''' </summary>
            ''' <returns></returns>
            <Xml.Serialization.XmlElement("方向")>
            Public Property Direction As Directions = Directions.Ascending

        End Class


        '----------------------------------------------------------------------------------------------------------------------------


        ''' <summary>
        ''' 分類に関する設定値
        ''' </summary>
        Public Class ClassifyParameter

            ''' <summary>
            ''' 色分けするか
            ''' </summary>
            ''' <returns></returns>
            <Xml.Serialization.XmlElement("色分け")>
            Public Property ColorCoded As Boolean = True

            ''' <summary>
            ''' 格納アイテムの背景色
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("格納アイテム色")>
            Public Property DirectoryItemColor As New SerializableColor With {.Color = Color.FromArgb(&H1F, Color.Yellow)}

            ''' <summary>
            ''' 実行アイテムの背景色
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("実行アイテム色")>
            Public Property FileItemColor As New SerializableColor With {.Color = Color.FromArgb(&H5F, Color.White)}

        End Class

    End Class



    '============================================================================================================================



    ''' <summary>
    ''' 裏メニューに関する設定値
    ''' </summary>
    ''' <remarks></remarks>
    Public Class SecretMenuParameter

        ''' <summary>
        ''' メニュー項目の設置
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("メニュー設置")>
        Public Property MenuItemVisible As Boolean = True
        '※ 非表示にできる理由 … 秘匿性を重視した機能であるが故「開閉状況すら知られたくない」場合に対応するため。

        ''' <summary>
        ''' マウスのホイールに関する設定
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <System.Xml.Serialization.XmlElement("マウスホイール")>
        Public Property MouseWheel As New MouseWheelParameter


        '----------------------------------------------------------------------------------------------------------------------------


        ''' <summary>
        ''' マウスのホイールに関する設定値
        ''' </summary>
        ''' <remarks></remarks>
        Public Class MouseWheelParameter

            ''' <summary>
            ''' 無段ホイールモードを使用する
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("無段ホイールモード")>
            Public Property SlipWheelMode As Boolean = True

            ''' <summary>
            ''' 回転を分離する閾時間
            ''' </summary>
            ''' <value></value>
            ''' <returns></returns>
            ''' <remarks></remarks>
            <System.Xml.Serialization.XmlElement("回転分離閾時間")>
            Public Property RollSeparateTime As Integer = 250

            ''' <summary>
            ''' パラメータの複製を作る(一般的な《Clone》メソッドと同じ）。
            ''' </summary>
            ''' <returns></returns>
            ''' <remarks></remarks>
            Public Overloads Function Copy() As MouseWheelParameter

                Return New MouseWheelParameter With {.SlipWheelMode = SlipWheelMode, .RollSeparateTime = RollSeparateTime}

            End Function

            ''' <summary>
            ''' パラメータの複製を作る。
            ''' </summary>
            ''' <param name="TemporarySlipWheelMode">一時的に使用する無段ホイールモード</param>
            ''' <returns></returns>
            ''' <remarks></remarks>
            Public Overloads Function Copy(TemporarySlipWheelMode As Boolean) As MouseWheelParameter

                Return New MouseWheelParameter With {.SlipWheelMode = TemporarySlipWheelMode, .RollSeparateTime = RollSeparateTime}

            End Function

        End Class

    End Class



    '============================================================================================================================



    ''' <summary>
    ''' シリアライズ可能なColor型
    ''' </summary>
    ''' <remarks></remarks>
    Public Class SerializableColor

        'Color構造体は、XMLに出力されない（項目名だけ書き出される）ので、文字列型としてカラーコードを読み書きする。

        ''' <summary>
        ''' 色
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <Xml.Serialization.XmlIgnore()>
        Public Property Color As Color = Color.White

        ''' <summary>
        ''' ARGB32プロパティの基数
        ''' </summary>
        ''' <returns></returns>
        <Xml.Serialization.XmlAttribute("基数")>
        Public Property Base As Integer

            Get
                Return _Base
            End Get

            Set
                Select Case Value
                    Case 2, 8, 10, 16
                        _Base = Value
                    Case Else
                        _Base = 16
                End Select
            End Set

        End Property
        Private _Base As Integer = 16

        ''' <summary>
        ''' 32bitのARGB値
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        <Xml.Serialization.XmlText()>
        Public Property ARGB32 As String

            Get
                Return Convert.ToString(Color.ToArgb, Base)
            End Get

            Set(value As String)
                Try
                    Color = Color.FromArgb(Convert.ToInt32(value, Base))
                Catch ex As Exception
                    Color = Color.FromArgb(0)
                End Try
            End Set

        End Property

    End Class

End Class
