﻿Imports System.ComponentModel



''' <summary>
''' 色選択
''' </summary>
Public Class ColorBox

    Private Chip As Bitmap '色サンプル描画用ビットマップ
    Private RGB As Integer '現在のRGB値（A値除く）


    ''' <summary>
    ''' 色が選択された
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="selectColor"></param>
    Public Event ColorSelected(sender As Object, selectColor As Color)



    ''' <summary>
    ''' 現在の色を取得・設定する
    ''' </summary>
    ''' <returns></returns>
    <Category("動作"), Description("現在の色を取得・設定する。")>
    Public Property Color As Color
        Get
            Return Color.FromArgb((CInt(nudOpecity.Value) << 24) Or RGB)
        End Get
        Set(value As Color)
            SetColor(value)
        End Set
    End Property

    ''' <summary>
    ''' 色コレクションを取得・設定する
    ''' </summary>
    ''' <returns></returns>
    <Category("動作"), Description("色コレクションを取得・設定する。")>
    Public Property Collections As Integer()



    Private Sub ColorBox_Paint(sender As Object, e As PaintEventArgs) Handles Me.Paint

        '=== 色チップを再描画 ===
        btnPicker.Refresh()
        '※ ユーザーコントロールの更新時にボタンも更新しないと、色チップが描き変わらない。
        '※ 逆に言えば、色チップを更新したければ、ユーザーコントロールを更新しなければならない。

    End Sub

    Private Sub btnPicker_Paint(sender As Object, e As PaintEventArgs) Handles btnPicker.Paint

        '=== 色サンプルを更新 ===
        Using grp As Graphics = Graphics.FromImage(Chip) '絵具
            With grp
                Dim rect = New Rectangle(0, 0, Chip.Width - 1, Chip.Height - 1)
                .FillRectangle(New SolidBrush(Color.FromArgb(RGB Or &HFF000000)), rect)
                .DrawRectangle(Pens.Black, rect)
            End With
        End Using
        btnPicker.Image = Chip

    End Sub

    Private Sub btnPicker_SizeChanged(sender As Object, e As EventArgs) Handles btnPicker.SizeChanged

        '=== ボタン寸法に色チップを合わせる ===
        With btnPicker
            Chip = New Bitmap(.Width \ 3, .Height \ 2)
        End With

    End Sub

    Private Sub btnPicker_Click(sender As Object, e As EventArgs) Handles btnPicker.Click

        '=== 色を選択 ===
        Using picker As New ColorDialog With {.Color = Color, .FullOpen = True, .CustomColors = Collections}
            With picker
                If .ShowDialog(Me) = DialogResult.OK Then
                    '--- 色コレクションを更新 ---
                    '※ 他の色選択ダイアログと共有しているので、配列ごと代入することはできない。
                    For i = 0 To Collections.Length - 1
                        Collections(i) = .CustomColors(i)
                    Next
                    'ちょっとしたメモ的に便利かなと思って実装したのだけど…
                    'ダイアログでキャンセルすると、CustomColorsに加えた変更が無かったことにされるので、残念ながらあまり便利になってない。
                    'そもそもダイアログは、中で行った選択結果と、最終的に行ったOK・キャンセル結果の両者を返し、その結果をどう扱うかはプログラマに委ねられるべきではないのか？
                    'キャンセルをクリックしたからと言って、ユーザーが行った操作を無にして良いのだろうか？

                    '--- 色を更新 ---
                    SetColor(.Color, False)
                    RaiseEvent ColorSelected(Me, .Color)
                End If
            End With
        End Using

    End Sub

    ''' <summary>
    ''' 現在の色を設定する
    ''' </summary>
    ''' <param name="color">新しい色</param>
    ''' <param name="withA">アルファ値書き換え許可</param>
    Private Sub SetColor(color As Color, Optional withA As Boolean = True)

        With color
            If withA Then nudOpecity.Value = .A
            RGB = .ToArgb And &HFFFFFF

            ToolTip1.SetToolTip(btnPicker, $"R = { .R:d3} ({ .R:X2}){Environment.NewLine}G = { .G:d3} ({ .G:X2}){Environment.NewLine}B = { .B:d3} ({ .B:X2})")
        End With

        Refresh()

    End Sub

    ''' <summary>
    ''' ColorからBGR24に変換する
    ''' </summary>
    ''' <param name="color">素の色</param>
    ''' <returns>BGR値</returns>
    Public Shared Function ColorToBGR24(color As Color) As Integer

        With color
            Return .R Or (CInt(.G) << 8) Or (CInt(.B) << 16)
        End With

        '※ ColorDialog.CustomColorsで指定する色は、BGRになっているみたい。VB6時代の名残かな？

    End Function

    Private Sub chkBase_CheckedChanged(sender As Object, e As EventArgs) Handles chkBase.CheckedChanged

        '=== 基数を変更 ===
        nudOpecity.Hexadecimal = chkBase.Checked

    End Sub

End Class
