﻿'┌────┐
'│技術要綱├──────────────────────
'└┬───┘
'  │『終了コードを返す方法』
'  │※本作では終了コードを、設定コンソールがどのように閉じられたのかを呼び出し元に知らせる手段として用いている。
'  │  終了コードは、本来異常終了した時のエラーコードを返すのに用いられるのだと思う（コンソール・アプリケーションだけの話なのかも？）が、ここに自分勝手な数値を放り込むことで、システムにどのような影響があるのかは分からない。そもそも、こういう用途に利用していい機能なの？ OS自身は、このコードに関与しているの？
'  │○作成手順
'  │・“エントリポイント”を《Function》で宣言して、《Return》で返す。
'  │↓
'  │・“エントリポイント”を自作しなければならない（最近のVBは、《MyApplication》クラスに《Main》メソッドが存在する（プロパティの[アプリケーション イベントの表示]ボタンで出現）が、ユーザにエントリポイントを弄らせてくれない）。
'  ││└→ 1.プロジェクトのプロパティの[[アプリケーション]]タブにある[アプリケーション フレームワークを有効にする]のチェックを外す。
'  ││     2.モジュールを追加してエントリポイントを記述する（《Main》）。
'  ││     3.[スタートアップ オブジェクト]からエントリーポイントを記述したモジュールを選択する。
'  │↓
'  │・“エントリポイント”を自作すると、二重起動を禁止する機能（[Windows アプリケーション フレームワーク プロパティ]の[単一インスタンスのアプリケーションを作成する]）が使えなくなるので、こちらも自分で書く必要がある。
'  └────────────



''' <summary>
''' 設定コンソールのエントリポイント記述場所
''' </summary>
''' <remarks></remarks>
Module Program

    ''' <summary>終了コード番号</summary>
    Public Enum ExitCodeID As Integer

        ''' <summary>パラメータ不整合（エラー）</summary>
        PrameterError = -2

        ''' <summary>二重起動（エラー）</summary>
        DoubleBoot

        ''' <summary>キャンセル</summary>
        Cancel = 0

        ''' <summary>OK</summary>
        OK

        ''' <summary>適用</summary>
        Apply

    End Enum


    ''' <summary>終了コード</summary>
    Public ExitCode As ExitCodeID = ExitCodeID.Cancel


    '::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


    ''' <summary>
    ''' エントリポイント
    ''' </summary>
    ''' <returns>終了コード</returns>
    ''' <remarks></remarks>
    <STAThread()> _
    Function Main() As Integer

        '=== 二重起動チェックと起動 ===
        Dim createdNew As Boolean
        Dim mutex As New Threading.Mutex(True, My.Application.Info.ProductName, createdNew)
        If createdNew = False Then 'ミューテックスの初期所有権が付与されなかったら、起動済みと判断できるそうです。

            '--- 起動済み ---
            MessageBox.Show("既に起動しています。", System.IO.Path.GetFileName(System.Windows.Forms.Application.ExecutablePath), MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            ExitCode = ExitCodeID.DoubleBoot

        Else

            '--- 起動 ---
            Application.EnableVisualStyles()
            Application.SetCompatibleTextRenderingDefault(False)
            Application.Run(New FormOption()) '起動済み

            '--- ミューテックスを解放 ---
            mutex.ReleaseMutex() '終了時にこれをやっとかないと、不都合があるみたい。二重起動で止めるときには要らないみたい（エラーになる）。

        End If

        '=== 終了コードを返す ===
        Debug.WriteLine(String.Format("終了コード:Send = {0}({1})", ExitCode.ToString, CInt(ExitCode)))
        Return ExitCode

    End Function

End Module





'||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||





''' <summary>
''' 設定フォーム
''' </summary>
''' <remarks></remarks>
Public Class FormOption

    Private Sub FormSetup_Load(sender As System.Object, e As System.EventArgs) Handles MyBase.Load

        '=== 基本設定 ===
        With My.Application

            'バージョン情報
            picIcon.Image = Me.Icon.ToBitmap
            picLogo.Image = Setting.My.Resources.DWs
            With .Info
                lblTitle.Text = $"『{ .Title}』"
                lblVersion.Text = "Version " & .Version.ToString
                lblCopylight.Text = .Copyright
                lblCompany.Text = .CompanyName
            End With

            '--- コマンドライン引数 ---
            '"-○○"系パラメータは、ランチャ本体からの指示に使われる。

            'タイトル指定（立ち上げ饂飩から呼び出されたことを示すフラグを兼ねる。）
            Dim titleIndex As Integer = .CommandLineArgs.IndexOf("-title")
            If titleIndex > -1 AndAlso titleIndex < .CommandLineArgs.Count - 1 Then '"-title"の記述位置が最後だったら、パラメータが足りないということ。
                Me.Text = $"{ .Info.Title} - { .CommandLineArgs(titleIndex + 1)}"
            Else
                Me.Text = .Info.Title
                MessageBox.Show("設定コンソールを直接起動して変更した内容は、設定ファイルへは保存されますが、『立ち上げ饂飩』には反映されません。" & vbCrLf & "反映させるには、『立ち上げ饂飩』を再起動するか、『立ち上げ饂飩』のシステムメニューから[設定]を選択し[適用]ボタンをクリックする必要があります。", Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Information)
            End If

            '既に設定ファイルが更新・変更されていた
            If .CommandLineArgs.Contains("-modified") Then
                btnApply.Visible = True
                MessageBox.Show("設定ファイルが、既に変更されています。" & vbCrLf & "反映させるには、[適用]ボタンをクリックしてください。", Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Information)
            End If

            '錠前破りモード
            If .CommandLineArgs.Contains("/lockbreak") Then
                btnLockBreak.Visible = True
                TabControl1.SelectedTab = tabConsole
            End If

        End With

        '=== 設定パラメータの読み込み ===
        SettingsLoader()

        '=== 設定コントロールの準備 ===
        '--- タスクトレイ用アイコン ---
        With cmbIcon
            .DisplayMember = "caption"
            .ValueMember = "type"
            .DataSource = {
                New With {
                    .type = Setting.SystemParameter.TaskTrayParameter.IconParameter.Types.Steam,
                    .caption = Setting.SystemParameter.TaskTrayParameter.IconParameter.GetName(.type),
                    .image = Setting.SystemParameter.TaskTrayParameter.IconParameter.GetImage(.type)
                },
                New With {
                    .type = Setting.SystemParameter.TaskTrayParameter.IconParameter.Types.Rise,
                    .caption = Setting.SystemParameter.TaskTrayParameter.IconParameter.GetName(.type),
                    .image = Setting.SystemParameter.TaskTrayParameter.IconParameter.GetImage(.type)
                },
                New With {
                    .type = Setting.SystemParameter.TaskTrayParameter.IconParameter.Types.Macho,
                    .caption = Setting.SystemParameter.TaskTrayParameter.IconParameter.GetName(.type),
                    .image = Setting.SystemParameter.TaskTrayParameter.IconParameter.GetImage(.type)
                },
                New With {
                    .type = Setting.SystemParameter.TaskTrayParameter.IconParameter.Types.Rocket,
                    .caption = Setting.SystemParameter.TaskTrayParameter.IconParameter.GetName(.type),
                    .image = Setting.SystemParameter.TaskTrayParameter.IconParameter.GetImage(.type)
                }
            }
        End With
        txtTitle.MaxLength = Setting.SystemParameter.TaskTrayParameter.IconParameter.TitleMaxLength

        '--- 各カラーピッカーの色コレクション ---
        Dim tcc = Enumerable.Repeat(ColorBox.ColorToBGR24(Color.LightSkyBlue), 16).ToArray,
            tdc = New Setting.SystemParameter.TaskTrayParameter
        tcc(2) = ColorBox.ColorToBGR24(tdc.HoverPopupper.ActiveColor.Color)
        tcc(10) = ColorBox.ColorToBGR24(tdc.AutoCloser.ActiveColor.Color)
        cbxHover.Collections = tcc
        cbxClose.Collections = tcc

        Dim lcc = Enumerable.Repeat(ColorBox.ColorToBGR24(SystemColors.Menu), 16).ToArray,
            ldc = New Setting.SystemParameter.LaunchMenuParameter.ClassifyParameter
        lcc(2) = ColorBox.ColorToBGR24(ldc.DirectoryItemColor.Color)
        lcc(10) = ColorBox.ColorToBGR24(ldc.FileItemColor.Color)
        cbxDirItemColor.Collections = lcc
        cbxFileItemColor.Collections = lcc

        '色コレクションの配置
        '[0][1][2][3][4][5][6][7]
        '[8][9][A][B][C][D][E][F]
        '0,1,8,9 -> 自由枠
        '2,A     -> 既定色
        '3-7,B-F -> 履歴 (3,B <- 過去/現在 -> 7,F)

        '=== 現在値の反映 ===
        Setup()

        '=== ヘルプ文を登録 ===
        With hlpExplain
            Dim m0 = "ここにチェックを入れると、{0}ます。",
                m1 = $"{{0}}の色を指定します。{vbCrLf}A値で不透明度を調整できます。"
            Dim t0, t1 As String

            .SetHelpString(btnCancel, "変更内容を破棄して、設定を終了します。")
            .SetHelpString(btnOK, $"設定内容を確定して保存します。{vbCrLf}※鍵に変更がある場合は、確認する手続きが行われます。")
            .SetHelpString(btnApply, "この設定コンソールの内容を破棄して、現在の設定ファイルの内容を『立ち上げ饂飩』に適用します。")

            '--- タスクトレイ ---
            .SetHelpString(cmbIcon, "タスクトレイに表示するアイコンの種類を選択します。")
            .SetHelpString(txtTitle, $"タスクトレイのアイコンに表示する題名を付けます。{vbCrLf}※最大{txtTitle.MaxLength}文字。{vbCrLf}※空白文字にすると、アプリ名が表示されます。")

            t0 = "・待機時間を0ミリ秒にすることで機能を停止できます。"
            .SetHelpString(staHover, $"タスクトレイのアイコンにマウスカーソルを乗せてランチメニューが開くまでの待機時間と、その計測間隔を指定します。{vbCrLf}{t0}タスクトレイのアイコンをクリックして開いてください。")
            .SetHelpString(staClose, $"自動的にランチメニューが閉じられるまでの待機時間と、その計測間隔を指定します。{vbCrLf}{t0}{vbCrLf}・{grpHoverPopupper.Text}した時に作動します。{vbCrLf}・メニュー上にマウスカーソルがある間は、計測されません。{vbCrLf}・作動中にキーボード操作を行うと、計測を停止します。")

            t0 = $"{{0}}時の経過時間を表すゲージ"
            .SetHelpString(cbxHover, String.Format(m1, String.Format(t0, grpHoverPopupper.Text)))
            .SetHelpString(cbxClose, String.Format(m1, String.Format(t0, grpAutoCloser.Text)))

            '--- ランチメニュー ---
            .SetHelpString(txtRegistryPath, $"ランチメニューにアイテムを登録・配置するためのフォルダを指定します。{vbCrLf}※ローカル且つフルパスで指定してください。URI,UNCには対応していません。{vbCrLf}※旧フォルダの削除や登録内容の移行等は、別途行ってください。")
            .SetHelpString(btnChoicePath, $"フォルダを一覧から選択します。{vbCrLf}新規作成もできます。")

            .SetHelpString(nudLumpDepth, $"ランチメニューを構築する時、その対象となるメニューより下へ何層分まとめて構築するかを指定します。{If(SystemSettings.LaunchMenu.MenuBuild.RecursiveCallMax = 0, "", $"{vbCrLf}・ 0 なら、展開される層のアイテムのみを構築するので、読み込み時間を分散できます。{vbCrLf}・ 1{If(SystemSettings.LaunchMenu.MenuBuild.RecursiveCallMax = 1, "", $"～{SystemSettings.LaunchMenu.MenuBuild.RecursiveCallMax}")} なら、それだけ先読みして構築されるので、深い層へもいちいち待たされません。")}")
            .SetHelpString(chkStartUpBuild, String.Format(m0, "起動と共にランチメニューを構築するようになり") & $"{vbCrLf}※スタートアップと併用する場合、OSの起動に影響するかもしれません。")

            .SetHelpString(chkCloseItem, String.Format(m0, $"ランチメニューの最上層に[{Setting.My.Resources.CloseItem}]項目が表示され、手動で閉じられるようになり"))
            .SetHelpString(chkBatchRun, String.Format(m0, $"ランチメニューの各層に[{Setting.My.Resources.BatchRunItem}]項目が表示され、そこに格納されている全実行アイテムを実行できるようになり"))
            .SetHelpString(chkOpenSubDirectory, String.Format(m0, $"ランチメニューの各層に[{Setting.My.Resources.OpenRegistryDirectoryItem}]項目が表示され、その層に該当するランチメニュー登録フォルダを開けるようになり"))

            '--- アイテム ---
            t0 = "ランチメニューに並ぶアイテムを、{0}""{1}順""にします。"
            t1 = "名前を基に"
            .SetHelpString(rdoDictionaryOrder, String.Format(t0, t1, "辞書"))
            .SetHelpString(rdoNaturalOrder, String.Format(t0, t1, "自然") & vbCrLf & "※応答しなくなる場合は、他の並べ順に切り替えてください。")
            .SetHelpString(rdoCreateOrder, String.Format(t0, "", "作成日時"))
            .SetHelpString(rdoWriteOrder, String.Format(t0, "", "変更日時"))

            .SetHelpString(rdoAscendingOrder, String.Format(t0, "", "昇"))
            .SetHelpString(rdoDescendingOrder, String.Format(t0, "", "降"))

            .SetHelpString(chkColorCoded, String.Format(m0, "ランチメニューが種類ごとに着色され"))

            .SetHelpString(cbxDirItemColor, String.Format(m1, "格納アイテム"))
            .SetHelpString(cbxFileItemColor, String.Format(m1, "実行アイテム"))

            .SetHelpString(chkLargeIcon, String.Format(m0, "ランチメニューに表示されるアイコンが大きくなり") & If(chkLargeIcon.ThreeState, $"{vbCrLf}※不確定状態の場合は、現在値が維持されます。", ""))
            .SetHelpString(chkShowExtension, String.Format(m0, "実行アイテム名に拡張子が付き"))
            .SetHelpString(chkExCommandLineOption, String.Format(m0, "コマンドライン引数モードをマウスボタンで選択できるようになり") & $"{vbCrLf}・中クリック → オプション編集{vbCrLf}・右クリック → クリップボード転送{New String(vbCrLf, 2)}※修飾キー(Shift,Ctrl)によるモード選択は、この設定に影響されません。")

            '--- 裏メニュー ---
            .SetHelpString(chkMenuVisible, String.Format(m0, "システムメニューに[裏メニュー]項目が表示され、裏メニューの開示切り替えと現況確認ができるようになり"))

            t0 = Chr(&H8167) '全角ダブルクォーテーション（前）[“]
            t1 = Chr(&H8168) '全角ダブルクォーテーション（後）[”]
            .SetHelpString(chkKeyChange, $"鍵の 作成・変更・解除 を行いたい時に、チェックを入れます。{vbCrLf}その際、既に鍵が設定されている場合は、入力を求められます。{vbCrLf}変更を取り止めたい時は、チェックを外してください。{vbCrLf}{vbCrLf}鍵には、{t0}ダイヤル{t1}と{t0}合言葉{t1}の2種類があり、両方でも片方だけでも利用できます。{vbCrLf}{vbCrLf}設定した鍵は、裏メニューを開く（ランチメニューの裏アイテムを表示する）時に必要になります。")
            .SetHelpString(dbtnNewDial, $"{t0}ダイヤル{t1}は、マウスのホイールの回転パターンを鍵に用います。{vbCrLf}設定する場合は、ここに新しいパターンを、{dbtnNewDial.StepMaxLength}ステップまでで入力します。{vbCrLf}{vbCrLf}入力には、マウスのホイールまたはキーボードの[Page Up]/[Page Down]キーを使用します。{vbCrLf}入力を取り消したい時は、クリックしてください。{vbCrLf}{vbCrLf}※『未入力』のまま確定すると、解除されます。合言葉のみの変更であっても、現行ダイヤルの再入力が必要です。{vbCrLf}※マウスカーソルが乗ると強制的にフォーカスを奪うので、他の項目を操作している時は注意してください。{vbCrLf}※ホイールについては、[無段ホイール モード]のヘルプも参照してください。")
            .SetHelpString(txtNewPassword, $"{t0}合言葉{t1}は、好きなパスワードを鍵に用います。{vbCrLf}設定する場合は、ここに新しい文字列を、半角{txtNewPassword.MaxLength}文字までで入力します。{vbCrLf}{vbCrLf}※『空欄』のまま確定すると、解除されます。{vbCrLf}ダイヤルのみの変更であっても、現行の合言葉を再度入力する必要があります。")
            .SetHelpString(chkSlipWheel, $"ダイヤルの入力に、ノッチ（回転時に{t0}カチッカチッ{t1}となる引っ掛かり）がない{vbCrLf}無段ホイールを使用する場合にチェックを入れます。{vbCrLf}また、より手軽に入力したい場合にも有効です。{vbCrLf}{vbCrLf}この設定によって、入力する感覚が以下の様に変化します。{vbCrLf}{vbCrLf}[チェック有り]{vbCrLf}一方向に連続する回転をまとめて1ステップとして認識します。{vbCrLf}※{SystemSettings.SecretMenu.MouseWheel.RollSeparateTime}ミリ秒以上間隔が空くと、別のステップとなります。{vbCrLf}「上下上上下…」という風に入力したい時に向く。{vbCrLf}{vbCrLf}[チェック無し]{vbCrLf}1ノッチを1ステップとして認識します。{vbCrLf}「下に2 上に3 下に1 …」という風に入力したい時に向く。")
        End With

    End Sub

    Private Sub btnOK_Click(sender As System.Object, e As System.EventArgs) Handles btnOK.Click

        '=== 確定 ===
        Dim keyChangeFlag As Boolean = False '鍵変更の意思 & 認可フラグ

        '--- 整合性チェック ---
        'ランチメニュー登録フォルダ
        Dim msg = Setting.SystemParameter.LaunchMenuParameter.JustifyPathCheck(txtRegistryPath.Text)
        If Not String.IsNullOrEmpty(msg) Then

            MessageBox.Show(Me, msg, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)

            TabControl1.SelectedTab = tabLaunchMenu
            With txtRegistryPath
                .Text = SystemSettings.LaunchMenu.RegistryPath
                .Focus()
                .SelectAll()
            End With

            Exit Sub '※※※

        End If

        '鍵
        If chkKeyChange.Checked Then '鍵の変更意思があるか？

            If dbtnNewDial.StepPattern = "" AndAlso txtNewPassword.Text = "" Then

                '鍵の解除確認
                keyChangeFlag = (MessageBox.Show("鍵が、解除されます。よろしいですか？", Me.Text & " - [鍵の解除]",
                                                 MessageBoxButtons.OKCancel,
                                                 MessageBoxIcon.Exclamation,
                                                 MessageBoxDefaultButton.Button2) = DialogResult.OK) '《MessageBoxIcon.Question》を使う予定が、「使うな」ってことなので。

            Else

                '新鍵の確認
                Dim tKey As New KeyParameter With {.Dial = dbtnNewDial.StepPattern, .Password = txtNewPassword.Text}
                Dim wheel As Setting.SystemParameter.SecretMenuParameter.MouseWheelParameter = SystemSettings.SecretMenu.MouseWheel.Copy(chkSlipWheel.Checked)

                Using inputKeyForm As New KeyInputBox.FormKey(tKey, wheel, "鍵が、変更されています。" & vbCrLf & "確認のため、新しい鍵をもう一度入力してください。", True) With {.Text = Me.Text & " - [鍵の変更]", .Icon = Me.Icon, .StartPosition = FormStartPosition.CenterParent}
                    keyChangeFlag = (inputKeyForm.ShowDialog(Me) = DialogResult.OK)
                End Using

            End If

            '必要なら設定に戻る
            If Not keyChangeFlag Then Exit Sub '※※※

        End If

        '--- パラメータに格納 ---
        With SystemSettings

            With .TaskTray
                With .Icon 'アイコン
                    .Type = cmbIcon.SelectedValue
                    .Title = txtTitle.Text
                End With

                With .HoverPopupper 'ホバー展開時間
                    .Time = staHover.Time
                    .AdjustStep = staHover.Interval
                    .ActiveColor.Color = cbxHover.Color
                End With

                With .AutoCloser '自動撤収
                    .Time = staClose.Time
                    .AdjustStep = staClose.Interval
                    .ActiveColor.Color = cbxClose.Color
                End With
            End With

            With .LaunchMenu
                .RegistryPath = txtRegistryPath.Text 'ランチメニュー登録フォルダ

                With .MenuBuild 'ランチメニューの構築
                    .StartUpBuild = chkStartUpBuild.Checked
                    .Depth = nudLumpDepth.Value
                End With

                .BatchRun = chkBatchRun.Checked '一括実行
                .SubDirectoryOpener = chkOpenSubDirectory.Checked 'ランチメニュー登録サブフォルダ展開
                .HandClose = chkCloseItem.Checked '手動撤収

                With .Sort '並べ替え
                    Select Case True
                        Case rdoDictionaryOrder.Checked
                            .Order = Setting.SystemParameter.LaunchMenuParameter.SortParameter.OrderTypes.Dictionary
                        Case rdoNaturalOrder.Checked
                            .Order = Setting.SystemParameter.LaunchMenuParameter.SortParameter.OrderTypes.Natural
                        Case rdoCreateOrder.Checked
                            .Order = Setting.SystemParameter.LaunchMenuParameter.SortParameter.OrderTypes.CreationTime
                        Case rdoWriteOrder.Checked
                            .Order = Setting.SystemParameter.LaunchMenuParameter.SortParameter.OrderTypes.LastWriteTime
                    End Select
                    .Direction = If(rdoDescendingOrder.Checked, Setting.SystemParameter.LaunchMenuParameter.SortParameter.Directions.Descending, Setting.SystemParameter.LaunchMenuParameter.SortParameter.Directions.Ascending)
                End With

                With .Classify '分類
                    .ColorCoded = chkColorCoded.Checked
                    .DirectoryItemColor.Color = cbxDirItemColor.Color
                    .FileItemColor.Color = cbxFileItemColor.Color
                End With

                Select Case chkLargeIcon.CheckState '大アイコン…CheckState.Indeterminateの場合は、値を変更しない。
                    Case CheckState.Unchecked
                        .IconImageScale = Setting.SystemParameter.LaunchMenuParameter.SmallIconScale
                    Case CheckState.Checked
                        .IconImageScale = Setting.SystemParameter.LaunchMenuParameter.LargeIconScale
                End Select
                .ShowExtension = chkShowExtension.Checked '拡張子表示
                .ExCommandLineOption = chkExCommandLineOption.Checked '拡張コマンドライン引数

            End With

            With .SecretMenu
                .MenuItemVisible = chkMenuVisible.Checked '裏メニュー項目の設置
                .MouseWheel.SlipWheelMode = chkSlipWheel.Checked '無段ホイール モード
            End With

        End With

        '--- 保存 ---
        'システム設定あらメータの書き込み
        Try
            SettingsSave(SystemSettingsFileName, SystemSettings)
        Catch ex As Exception
            MessageBox.Show("システム設定の保存中に例外が発生しました。" & vbCrLf & vbCrLf & ex.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try

        '鍵設定パラメータの書き出し
        If keyChangeFlag Then '変更許可があるときだけ書き出す。

            KeySettings.Password = txtNewPassword.Text
            KeySettings.Dial = dbtnNewDial.StepPattern

            Try
                SettingsSave(keySettingsFileName, KeySettings, True)
            Catch ex As Exception
                MessageBox.Show("鍵の保存中に例外が発生しました。" & vbCrLf & vbCrLf & ex.Message, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
            End Try

        End If

        '--- 閉じる ---
        ExitCode = ExitCodeID.OK
        Me.Close()

    End Sub

    Private Sub btnCancel_Click(sender As System.Object, e As System.EventArgs) Handles btnCancel.Click

        '=== キャンセル ===
        ExitCode = ExitCodeID.Cancel
        Me.Close()

    End Sub

    Private Sub btnApply_Click(sender As System.Object, e As System.EventArgs) Handles btnApply.Click

        '=== 適用 ===
        ExitCode = ExitCodeID.Apply
        Me.Close()

    End Sub

    Private Sub cmbIcon_SelectedValueChanged(sender As Object, e As EventArgs) Handles cmbIcon.SelectedValueChanged

        '=== アイコン画像を更新 ===
        picImage.Image = cmbIcon.SelectedItem?.image

    End Sub

    Private Sub ColorBox_ColorSelected(sender As Object, selectColor As Color) Handles cbxHover.ColorSelected, cbxClose.ColorSelected, cbxFileItemColor.ColorSelected, cbxDirItemColor.ColorSelected

        '=== 選択色の履歴を残す ===
        PushColorHistory(sender, selectColor)

    End Sub

    Private Sub btnChoicePath_Click(sender As System.Object, e As System.EventArgs) Handles btnChoicePath.Click

        '=== ランチメニュー登録フォルダを選択 ===
        Using fbd As New FolderBrowserDialog With {.Description = "新しい「ランチメニュー登録フォルダ」を選択してください。",
                                                   .SelectedPath = txtRegistryPath.Text}
            With fbd
                If .ShowDialog(Me) = DialogResult.OK Then
                    Dim msg = Setting.SystemParameter.LaunchMenuParameter.JustifyPathCheck(.SelectedPath)
                    If Not String.IsNullOrEmpty(msg) Then
                        MessageBox.Show(Me, msg, Me.Text, MessageBoxButtons.OK, MessageBoxIcon.Error)
                    Else
                        txtRegistryPath.Text = .SelectedPath
                    End If
                End If
            End With
        End Using

    End Sub

    Private Sub chkKeyChange_CheckedChanged(sender As Object, e As System.EventArgs) Handles chkKeyChange.CheckedChanged

        '=== 鍵変更の意思表明 ===
        With DirectCast(sender, CheckBox)

            If .Checked Then

                '--- 現行鍵の照合 ---
                If KeySettings.Exists Then

                    .CheckState = CheckState.Indeterminate '保留

                    '入力
                    Using inputKeyForm As New KeyInputBox.FormKey(KeySettings, SystemSettings.SecretMenu.MouseWheel.Copy(chkSlipWheel.Checked), "現在設定されている鍵を入力してください。") With {.Text = Me.Text & " - [鍵の変更]", .Icon = Me.Icon, .StartPosition = FormStartPosition.CenterParent}
                        .CheckState = IIf(inputKeyForm.ShowDialog(Me) = DialogResult.OK, CheckState.Checked, CheckState.Unchecked)
                    End Using

                End If

                '認証
                If .Checked Then
                    tlpNewKey.Enabled = True
                    btnOK.Text = "OK..."
                End If

            Else

                '--- 意思の取り下げ ---
                tlpNewKey.Enabled = False
                btnOK.Text = "OK"

            End If

        End With

    End Sub

    Private Sub chkSlipWheel_CheckedChanged(sender As Object, e As System.EventArgs) Handles chkSlipWheel.CheckedChanged

        '=== 無段ホイール モードの切り替え ===
        dbtnNewDial.SlipWheel = DirectCast(sender, CheckBox).Checked

    End Sub

    Private Sub dbtnNewDial_MouseIn(sender As Object, e As System.EventArgs) Handles dbtnNewDial.MouseIn

        '=== フォーカスを奪う前のアクティブコントロールを記録 ===
        dbtnNewDial.Tag = Me.ActiveControl

    End Sub

    Private Sub dbtnNewDial_MouseOut(sender As Object, e As System.EventArgs) Handles dbtnNewDial.MouseOut

        '=== フォーカスを奪う前のアクティブコントロールにフォーカスを返す ===
        Me.ActiveControl = DirectCast(dbtnNewDial.Tag, Control)

    End Sub

    Private Sub btnReload_Click(sender As Object, e As EventArgs) Handles btnReload.Click

        '=== 設定内容を再読み込み ===
        If MessageBox.Show(Me, "保存されている設定内容を読み込みます。" & Environment.NewLine & "※ 現在の状態は破棄されます。", Me.Text, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2) = DialogResult.OK Then
            SettingsLoader()
            Setup()
        End If

    End Sub

    Private Sub btnReset_Click(sender As Object, e As EventArgs) Handles btnReset.Click

        '=== 全項目初期化 ===
        If MessageBox.Show("全ての要素が初期化されます。" & Environment.NewLine & "※  設定を確定するまで、保存されません。", Me.Text, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2) = DialogResult.OK Then
            SystemSettings = New Setting.SystemParameter With {.Revision = Setting.SystemParameter.NOW_REVISION}
            Setup()
        End If

    End Sub

    Private Sub btnLockBreak_Click(sender As Object, e As EventArgs) Handles btnLockBreak.Click

        '=== 錠前破り ===
        If MessageBox.Show("現在の鍵の認証なしに、新しい鍵を作成できるようにします。" & Environment.NewLine & "※ 設定を確定する必要があります。", Me.Text, MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2) = DialogResult.OK Then
            KeySettings = New KeyParameter With {.Revision = KeyParameter.NOW_REVISION}
            chkKeyChange.Checked = True
            dbtnNewDial.StepPattern = ""
            txtNewPassword.Text = ""
            TabControl1.SelectedTab = tabSecretMenu
        End If

    End Sub

    ''' <summary>
    ''' 設定を読み込む
    ''' </summary>
    Private Sub SettingsLoader()

        '=== システム設定パラメータ ===
        Dim errorPoint As String = ""
        Try
            errorPoint = SettingsLoad(SystemSettingsFileName, SystemSettings)
        Catch ex As Exception
            MessageBox.Show("設定の読み込み・復元中に例外が発生しました。" & vbCrLf & "規定値が表示されるので、設定をやり直してください。" & vbCrLf & vbCrLf & ex.Message,
                                Me.Text,
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error) '読み込み中にエラーが発生した場合は、読み込めた部分も含め何も反映されないようなので、再初期化は不要。
        End Try

        '整合性エラーの警告
        If errorPoint <> "" Then
            errorPoint = errorPoint.Replace("//", vbCrLf).Replace("/", "").Replace("+", vbTab)
            MessageBox.Show("システム設定ファイルに記述されている下記の項目に関する設定値に致命的な不整合がありました。" & vbCrLf & "これらの項目は、設定コンソールから変更できないため動作を停止します。" & vbCrLf & vbCrLf & errorPoint & vbCrLf & vbCrLf & "設定値を書き直すか、ファイルを削除（初期化）して、システム設定ファイルの整合性を回復してください。",
                                Me.Text,
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Warning)

            ExitCode = ExitCodeID.PrameterError
            Me.Close() '継続断念※※※※
            Exit Sub
        End If

        '=== 鍵設定パラメータ ===
        Try
            SettingsLoad(keySettingsFileName, KeySettings)
        Catch ex As Exception
            MessageBox.Show("鍵の読み込み・復元中に例外が発生しました。" & vbCrLf & "解除された状態になりますので、作成し直してください。" & vbCrLf & vbCrLf & ex.Message,
                                Me.Text,
                                MessageBoxButtons.OK,
                                MessageBoxIcon.Error) '読み込み中にエラーが発生した場合は、読み込めた部分も含め何も反映されないようなので、再初期化は不要。
        End Try

    End Sub

    ''' <summary>
    ''' 色コレクションに履歴を残す
    ''' </summary>
    ''' <param name="target">対象ColorBox</param>
    ''' <param name="addColor">追加する色</param>
    Private Sub PushColorHistory(target As ColorBox, addColor As Color)

        With target

            '履歴部の基準要素番号を確定
            Dim n = 7 - 8 * {NameOf(cbxClose), NameOf(cbxFileItemColor)}.Contains(.Name)

            '履歴をずらす
            For i = n - 3 To n
                .Collections(i - 1) = .Collections(i)
            Next

            '新色を追加
            .Collections(n) = ColorBox.ColorToBGR24(addColor)

        End With

        '色コレクションの配置
        '[0][1][2][3][4][5][6][7]
        '[8][9][A][B][C][D][E][F]
        '0,1,8,9 -> 自由枠
        '2,A     -> 既定色
        '3-7,B-F -> 履歴 (3,B <- 過去/現在 -> 7,F)

    End Sub

    ''' <summary>
    ''' 設定ファイルの内容を反映する
    ''' </summary>
    Private Sub Setup()

        With SystemSettings

            '--- [タスクトレイ]タブ ---
            With .TaskTray

                'アイコン
                With .Icon
                    cmbIcon.SelectedValue = .Type
                    txtTitle.Text = .Title
                End With

                'ホバー展開
                With .HoverPopupper
                    staHover.Initialize(.Time, .AdjustStep, Setting.SystemParameter.TaskTrayParameter.TimeMeasureParameter.AdjustMax)
                    cbxHover.Color = .ActiveColor.Color
                    PushColorHistory(cbxHover, .ActiveColor.Color)
                End With

                '自動撤収
                With .AutoCloser
                    staClose.Initialize(.Time, .AdjustStep, Setting.SystemParameter.TaskTrayParameter.TimeMeasureParameter.AdjustMax)
                    cbxClose.Color = .ActiveColor.Color
                    PushColorHistory(cbxClose, .ActiveColor.Color)
                End With

            End With

            '--- [ランチメニュー]タブ ---
            With .LaunchMenu

                '登録フォルダ
                txtRegistryPath.Text = .RegistryPath

                'メニュー構築
                With .MenuBuild
                    chkStartUpBuild.Checked = .StartUpBuild
                    nudLumpDepth.Maximum = .RecursiveCallMax
                    nudLumpDepth.Value = .Depth
                    nudLumpDepth.Enabled = (.RecursiveCallMax > 0)
                End With

                'その他
                chkBatchRun.Checked = .BatchRun '一括起動
                chkOpenSubDirectory.Checked = .SubDirectoryOpener 'ランチメニュー登録サブフォルダ展開
                chkCloseItem.Checked = .HandClose '手動撤収

            End With

            '--- [アイテム]タブ ---
            With .LaunchMenu

                '並べ替え
                With .Sort
                    Select Case .Order
                        Case Setting.SystemParameter.LaunchMenuParameter.SortParameter.OrderTypes.Dictionary
                            rdoDictionaryOrder.Checked = True
                        Case Setting.SystemParameter.LaunchMenuParameter.SortParameter.OrderTypes.Natural
                            rdoNaturalOrder.Checked = True
                        Case Setting.SystemParameter.LaunchMenuParameter.SortParameter.OrderTypes.CreationTime
                            rdoCreateOrder.Checked = True
                        Case Setting.SystemParameter.LaunchMenuParameter.SortParameter.OrderTypes.LastWriteTime
                            rdoWriteOrder.Checked = True
                    End Select
                    Select Case .Direction
                        Case Setting.SystemParameter.LaunchMenuParameter.SortParameter.Directions.Ascending
                            rdoAscendingOrder.Checked = True
                        Case Setting.SystemParameter.LaunchMenuParameter.SortParameter.Directions.Descending
                            rdoDescendingOrder.Checked = True
                    End Select
                End With

                '分類
                With .Classify
                    chkColorCoded.Checked = .ColorCoded
                    cbxDirItemColor.Color = .DirectoryItemColor.Color
                    cbxFileItemColor.Color = .FileItemColor.Color
                    PushColorHistory(cbxDirItemColor, .DirectoryItemColor.Color)
                    PushColorHistory(cbxFileItemColor, .FileItemColor.Color)
                End With

                'その他
                Select Case .IconImageScale '大アイコン
                    Case Setting.SystemParameter.LaunchMenuParameter.SmallIconScale
                        chkLargeIcon.ThreeState = False
                        chkLargeIcon.CheckState = CheckState.Unchecked
                    Case Setting.SystemParameter.LaunchMenuParameter.LargeIconScale
                        chkLargeIcon.ThreeState = False
                        chkLargeIcon.CheckState = CheckState.Checked
                    Case Else
                        chkLargeIcon.ThreeState = True
                        chkLargeIcon.CheckState = CheckState.Indeterminate
                End Select
                chkShowExtension.Checked = .ShowExtension '拡張子表示
                chkExCommandLineOption.Checked = .ExCommandLineOption '拡張コマンドライン引数

            End With

            '--- [裏メニュー]タブ ---
            With .SecretMenu

                '裏メニュー項目の設置
                chkMenuVisible.Checked = .MenuItemVisible

                With .MouseWheel
                    '無段ホイール モード
                    chkSlipWheel.Checked = .SlipWheelMode

                    'ダイヤル
                    dbtnNewDial.StepPattern = ""
                    dbtnNewDial.SlipWheel = .SlipWheelMode
                    dbtnNewDial.RollSeparateTime = .RollSeparateTime
                End With

                '合言葉
                txtNewPassword.Text = ""

            End With

            '鍵変更スイッチ
            chkKeyChange.Checked = False

        End With

        '--- リビジョン表示 ---
        lblRevision.Text = $"Rev.{SystemSettings.Revision} / {KeySettings.Revision}"

    End Sub

End Class